@@ -1,4 +1,4 @@
-import datetime
+from datetime import timedelta
 # ---------------------------------------------------
 # Host configuration
 # ---------------------------------------------------
@@ -41,7 +41,8 @@ DEBUG = False
 # Database configuration
 # ---------------------------------------------------
 SQLALCHEMY_DATABASE_URI = 'sqlite:///:memory:'
-# If you are using a MySQL database, you must specify the URI as demonstrated below.
+# If you are using a MySQL database, you must specify the URI as
+# demonstrated below.
 # SQLALCHEMY_DATABASE_URI = 'mysql://username:password@hostname/database'
 SQLALCHEMY_ECHO = False
 SQLALCHEMY_TRACK_MODIFICATIONS = False
@@ -150,16 +151,17 @@ AUTH_PROVIDERS = []
 # adds the appropriate user identifier to the http headers of the request. If
 # enabled below, then they take precedence over any other forms of
 # authentication. If the call to `AUTH_MAP_REQUEST_HEADERS` results in a null
-# user identifier, then the authentication flow will fall back to use any of the
-# providers specified above.
+# user identifier, then the authentication flow will fall back to use any of
+# the providers specified above.
 AUTH_USE_REQUEST_HEADERS = False
 
 
-# If using headers to authenticate, the following function should be implemented
-# to transform a dictionary of headers into a dictionary of user attributes.
-# Currently only 'identifier', 'avatar_uri', 'name' and 'email' are supported.
-# If this method returns `None`, or `identifier` is not supplied, then the
-# authorization flow will fall back to other authentication methods.
+# If using headers to authenticate, the following function should be
+# implemented to transform a dictionary of headers into a dictionary of user
+# attributes. Currently only 'identifier', 'avatar_uri', 'name' and 'email'
+# are supported. If this method returns `None`, or `identifier` is not
+# supplied, then the authorization flow will fall back to other authentication
+# methods.
 def AUTH_MAP_REQUEST_HEADERS(headers):
     return {
         # 'identifier': None,
@@ -178,13 +180,13 @@ def AUTH_USER_IDENTIFIER_REQUEST_HEADER_MAPPING(identifier):
     return identifier
 
 
-# If the server desires to modify the attributes of the `User` object associated with
-# users logged in via any of the above authentication providers, it can do so via
-# this configuration key. This function will be run once at user login (if using
-# an `AuthenticationProvider`, and then at most once during any caching lifetime
-# period (as specified below). Note that attributes collected via
-# `AuthenticationProvider`s will not be updated after initial login (user must
-# relogin in order to reset those attributes).
+# If the server desires to modify the attributes of the `User` object
+# associated with users logged in via any of the above authentication
+# providers, it can do so via this configuration key. This function will be
+# run once at user login (if using an `AuthenticationProvider`, and then at
+# most once during any caching lifetime period (as specified below). Note that
+#  attributes collected via `AuthenticationProvider`s will not be updated
+# after initial login (user must relogin in order to reset those attributes).
 def AUTH_USER_ATTRIBUTE_SETTER(user):
     return user
 
@@ -198,7 +200,7 @@ AUTH_USER_ATTRIBUTE_CACHE_LIFETIME = 24 * 60 * 60  # 1 day
 # This is managed by `flask_login`, but is copied here for convenience.
 # For other options regarding sessions, please refer to:
 # https://flask-login.readthedocs.io/en/latest/#cookie-settings
-REMEMBER_COOKIE_DURATION = datetime.timedelta(days=365)
+REMEMBER_COOKIE_DURATION = timedelta(days=365)
 
 # ---------------------------------------------------
 # LDAP configuration
@@ -209,7 +211,8 @@ REMEMBER_COOKIE_DURATION = datetime.timedelta(days=365)
 
 # Currently the port and protocol must both be included in the server address
 LDAP_SERVER = 'ldap://127.0.0.1:389'
-# When entering this, note the "{0}" which denotes where the user_id is inserted.
+# When entering this, note the "{0}" which denotes where the user_id
+# is inserted.
 LDAP_USERDN_SCHEMA = 'cn={user_id},ou=people,dc=planetexpress,dc=com'
 
 # ---------------------------------------------------
@@ -267,30 +270,31 @@ def prepare_repo(repo):
 # Repo connected to the same database, accross multiple machines and/or
 # processes; and so a global indexing lock is employed. When a sync begins,
 # a sync lock is put in place and the responsible process is considered to be
-# the primary agent responsible for syncing until its last update is longer than
-# `INDEXING_TIMEOUT` seconds, whereby the lock is ceded to the next requesting
-# process. Note that `INDEXING_TIMEOUT` must be larger than `INDEXING_INTERVAL`
-# or strange things might begin to happen.
+# the primary agent responsible for syncing until its last update is longer
+# than`INDEXING_TIMEOUT` seconds, whereby the lock is ceded to the next
+# requesting process. Note that `INDEXING_TIMEOUT` must be larger than
+# `INDEXING_INTERVAL` or strange things might begin to happen.
 INDEXING_INTERVAL = 5 * 60  # 5 minutes
 INDEXING_TIMEOUT = 10 * 60  # 10 minutes
 
 # Whether an index operation should update repositories
 INDEXING_UPDATES_REPOSITORIES = True
 
-# Whether repositories should be updated even without a sync lock (in which case
-# the repositories will be updated on the sync timers, even if the relevant
-# process/thread does not have a lock on updating the index). This is useful in
-# context of multiple Knowledge Repo servers working together to serve the
-# repositories across multiple machines, which each require repository syncing.
-# Disable this if (for some reason) you have multiple Knowledge Repo servers
-# running on the same machine, and you want to avoid potential clashes. This
-# key is ignored if `INDEXING_UPDATES_REPOSITORIES` is False
+# Whether repositories should be updated even without a sync lock (in which
+# case the repositories will be updated on the sync timers, even if the
+# relevant process/thread does not have a lock on updating the index). This is
+# useful in context of multiple Knowledge Repo servers working together to
+# serve the repositories across multiple machines, which each require
+# repository syncing. Disable this if (for some reason) you have multiple
+# Knowledge Repo servers running on the same machine, and you want to avoid
+# potential clashes. This key is ignored if `INDEXING_UPDATES_REPOSITORIES` is
+# False
 INDEXING_UPDATES_REPOSITORIES_WITHOUT_LOCK = True
 
 # In some cases you may want to disable indexing entirely, which is currently
-# only ever used by the Knowledge Post previewer. Disabling the index means that
-# posts will not be discoverable, but if know the path in the repository you can
-# view the post with a direct link.
+# only ever used by the Knowledge Post previewer. Disabling the index means
+# that posts will not be discoverable, but if know the path in the repository
+# you can view the post with a direct link.
 INDEXING_ENABLED = True
 
 # ---------------------------------------------------
