@@ -1,13 +1,12 @@
+from .models import ErrorLog, IndexMetadata, Post
+from .proxies import db_session, current_app, current_repo
+from .utils.emails import send_subscription_emails
+from .utils.time import time_since
 import logging
 import multiprocessing
 import os
 import time
 
-from .proxies import db_session, current_repo, current_app
-from .models import ErrorLog, Post, IndexMetadata
-from .utils.emails import send_subscription_emails
-from .utils.time import time_since
-
 logging.basicConfig(level=logging.INFO)
 logger = logging.getLogger(__name__)
 
@@ -20,19 +19,28 @@ def set_up_indexing_timers(app):
     if not app.config['INDEXING_ENABLED']:
         return False
 
-    if app.check_thread_support(check_index=True, check_repositories=app.config['INDEXING_UPDATES_REPOSITORIES']):
+    if app.check_thread_support(check_index=True,
+                                check_repositories=app.config[
+                                    'INDEXING_UPDATES_REPOSITORIES'
+                                ]):
         if os.environ['KNOWLEDGE_REPO_MASTER_UUID'] != app.uuid:
-            logger.info("Not spawning index-sync timers for non-master application instance: {}".format(app.uuid))
+            logger.info(
+                f'Not spawning index-sync timers for non-master application \
+                instance: {app.uuid}')
             return
 
-        logger.info("Spawning index-sync timers for master application instance: {}".format(app.uuid))
+        logger.info(
+            f'Spawning index-sync timers for master application \
+                instance: {app.uuid}')
 
         def index_watchdog(app):
             while True:
                 if not hasattr(app, 'sync_thread') or not app.sync_thread.is_alive():
-                    logger.warning("Master indexing thread has died. Restarting...")
+                    logger.warning(
+                        "Master indexing thread has died. Restarting...")
                     with app.app_context():
-                        app.sync_thread = multiprocessing.Process(target=index_sync_loop, args=(app,))
+                        app.sync_thread = multiprocessing.Process(
+                            target=index_sync_loop, args=(app,))
                         app.sync_thread.start()
                 time.sleep(app.config['INDEXING_TIMEOUT'])
 
@@ -43,7 +51,8 @@ def set_up_indexing_timers(app):
                     update_index(check_timeouts=False)
                 time.sleep(app.config['INDEXING_INTERVAL'])
 
-        app.index_watchdog = multiprocessing.Process(target=index_watchdog, args=(app,))
+        app.index_watchdog = multiprocessing.Process(
+            target=index_watchdog, args=(app,))
         app.index_watchdog.start()
     else:
         @app.before_request
@@ -57,10 +66,22 @@ def acquire_index_lock():
             IndexMetadata.set('lock', 'master_check', current_app.uuid)
             db_session.commit()
             if (
-                IndexMetadata.get('lock', 'index_master') == current_app.uuid or
-                time_since(IndexMetadata.get_last_update('lock', 'index_master'), default=current_app.config["INDEXING_TIMEOUT"] + 1) > current_app.config["INDEXING_TIMEOUT"]
+                IndexMetadata.get(
+                    'lock',
+                    'index_master'
+                ) == current_app.uuid or time_since(
+                    IndexMetadata.get_last_update(
+                        'lock',
+                        'index_master'
+                    ),
+                    default=current_app.config[
+                        "INDEXING_TIMEOUT"
+                    ] + 1) > current_app.config[
+                        "INDEXING_TIMEOUT"
+                ]
             ):
-                IndexMetadata.set('lock', 'index_master', current_app.uuid)  # Update/set lock and update timestamp
+                # Update/set lock and update timestamp
+                IndexMetadata.set('lock', 'index_master', current_app.uuid)
                 return True
         finally:
             IndexMetadata.set('lock', 'master_check', None)
@@ -69,9 +90,15 @@ def acquire_index_lock():
 
 
 def is_indexing():
-    timeout = current_app.config["INDEXING_TIMEOUT"]  # Default index timeout to 10 minutes (after which indexing will be permitted to run again)
+    # Default index timeout to 10 minutes (after which indexing will be
+    # permitted to run again)
+    timeout = current_app.config["INDEXING_TIMEOUT"]
     last_update = time_since_index()
-    return IndexMetadata.get('lock', 'index', UNLOCKED) == LOCKED and last_update is not None and (last_update < timeout)
+    return IndexMetadata.get(
+        'lock',
+        'index',
+        UNLOCKED
+    ) == LOCKED and last_update is not None and (last_update < timeout)
 
 
 def time_since_index(human_readable=False):
@@ -108,7 +135,11 @@ def index_due_for_update():
     seconds = time_since_index()
     seconds_check = time_since_index_check()
 
-    if (seconds is not None and seconds_check is not None) and (seconds < interval or seconds_check < interval):
+    if (
+        seconds is not None and seconds_check is not None
+    ) and (
+            seconds < interval or seconds_check < interval
+    ):
         return False
 
     return True
@@ -136,7 +167,9 @@ def update_index(check_timeouts=True, force=False, reindex=False):
     # Check for update to repositories if configured to do so
     if (
         current_app.config['INDEXING_UPDATES_REPOSITORIES'] and
-        (is_index_master or current_app.config['INDEXING_UPDATES_REPOSITORIES_WITHOUT_LOCK'])
+        (is_index_master or current_app.config[
+            'INDEXING_UPDATES_REPOSITORIES_WITHOUT_LOCK'
+        ])
     ):
         current_repo.update()
 
@@ -154,37 +187,49 @@ def update_index(check_timeouts=True, force=False, reindex=False):
 
         for post in posts:
 
-            # If UUID has changed, check if we can find it elsewhere in the repository, and if so update index path
-            if post.uuid and ((post.path not in kr_dir) or (post.uuid != kr_dir[post.path].uuid)):
+            # If UUID has changed, check if we can find it elsewhere in the
+            # repository, and if so update index path
+            if post.uuid and (
+                    (post.path not in kr_dir) or (
+                        post.uuid != kr_dir[post.path].uuid
+                    )
+            ):
                 if post.uuid in kr_uuids:
-                    logger.info('Updating location of post: {} -> {}'.format(post.path, kr_uuids[post.uuid].path))
+                    logger.info(
+                        f'Updating location of post: {post.path} -> \
+                            {kr_uuids[post.uuid].path}')
                     post.path = kr_uuids[post.uuid].path
 
             # If path of post no longer in directory, mark as unpublished
             if post.path not in kr_dir:
-                logger.info('Recording unpublished status for post at {}'.format(post.path))
+                logger.info(
+                    f'Recording unpublished status for post at {post.path}')
                 post.status = current_repo.PostStatus.UNPUBLISHED
                 continue
 
-            # Update database according to current state of existing knowledge post and
-            # remove from kp_dir. This means that when this loop finishes, kr_dir will
-            # only contain posts which are new to the repo.
+            # Update database according to current state of existing
+            # knowledge post and remove from kp_dir. This means that
+            # when this loop finishes, kr_dir will only contain posts
+            # which are new to the repo.
             kp = kr_dir.pop(post.path)
 
             # Update metadata of post if required
-            if reindex or (kp.revision > post.revision or not post.is_published or kp.uuid != post.uuid):
+            if reindex or (
+                kp.revision > post.revision or not post.is_published or kp.uuid != post.uuid
+            ):
                 if kp.is_valid():
-                    logger.info('Recording update to post at: {}'.format(kp.path))
+                    logger.info(f'Recording update to post at: {kp.path}')
                     post.update_metadata_from_kp(kp)
                 else:
-                    logger.warning('Update to post at "{}" is corrupt.'.format(kp.path))
+                    logger.warning(
+                        f'Update to post at "{kp.path}" is corrupt.')
 
         # Add the new posts that remain in kr_dir
         for kp_path, kp in kr_dir.items():
             if not kp.is_valid():
-                logger.warning('New post at "{}" is corrupt.'.format(kp.path))
+                logger.warning(f'New post at "{kp.path}" is corrupt.')
                 continue
-            logger.info('creating new post from path {}'.format(kp_path))
+            logger.info(f'creating new post from path {kp_path}')
             post = Post()
             db_session.add(post)
             db_session.flush()  # (matthew) Fix groups logic so this is not necessary
