@@ -1,24 +1,22 @@
-import os
-import sys
-import datetime
-import logging
-import traceback
-from flask import current_app, request
-from flask_login import UserMixin
-from flask_sqlalchemy import SQLAlchemy
-from collections import defaultdict
-
-from sqlalchemy import func, distinct, and_, select, Index, UniqueConstraint
-
-from knowledge_repo._version import __version__
-from knowledge_repo.repository import KnowledgeRepository
-from knowledge_repo.utils.types import MediumText
 from .proxies import current_user, current_repo, db_session
 from .utils.models import unique_constructor
 from .utils.search import get_keywords
+from collections import defaultdict
+from datetime import datetime
+from flask import current_app, request
+from flask_login import UserMixin
+from flask_sqlalchemy import SQLAlchemy
+from knowledge_repo._version import __version__
+from knowledge_repo.repository import KnowledgeRepository
+from knowledge_repo.utils.types import MediumText
+from sqlalchemy import and_, distinct, func, select, Index
+from sqlalchemy.ext.associationproxy import association_proxy
 from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy.ext.orderinglist import ordering_list
-from sqlalchemy.ext.associationproxy import association_proxy
+import os
+import sys
+import logging
+import traceback
 
 logger = logging.getLogger(__name__)
 
@@ -32,28 +30,33 @@ class IndexMetadata(db.Model):
     type = db.Column(db.String(255), nullable=False)
     name = db.Column(db.String(512), nullable=False)
     value = db.Column(db.String(512), nullable=True)
-    updated_at = db.Column(db.DateTime, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow)
+    updated_at = db.Column(
+        db.DateTime, default=datetime.utcnow, onupdate=datetime.utcnow)
 
     @classmethod
     def get(cls, type, name, default=None):
-        m = db_session.query(IndexMetadata).filter(IndexMetadata.type == type).filter(IndexMetadata.name == name).first()
+        m = db_session.query(IndexMetadata).filter(
+            IndexMetadata.type == type).filter(IndexMetadata.name == name).first()
         if m is not None:
             return m.value
         return default
 
     @classmethod
     def set(cls, type, name, value):
-        m = db_session.query(IndexMetadata).filter(IndexMetadata.type == type).filter(IndexMetadata.name == name).first()
+        m = db_session.query(IndexMetadata).filter(
+            IndexMetadata.type == type).filter(IndexMetadata.name == name).first()
         if m is not None:
             m.value = value
-            m.updated_at = datetime.datetime.utcnow()
+            m.updated_at = datetime.utcnow()
         else:
-            m = IndexMetadata(type=type, name=name, value=value, updated_at=datetime.datetime.utcnow())
+            m = IndexMetadata(type=type, name=name, value=value,
+                              updated_at=datetime.utcnow())
             db_session.add(m)
 
     @classmethod
     def get_last_update(cls, type, name):
-        m = db_session.query(IndexMetadata).filter(IndexMetadata.type == type).filter(IndexMetadata.name == name).first()
+        m = db_session.query(IndexMetadata).filter(
+            IndexMetadata.type == type).filter(IndexMetadata.name == name).first()
         if m is not None:
             return m.updated_at
         return None
@@ -62,8 +65,10 @@ class IndexMetadata(db.Model):
 class PostAuthorAssoc(db.Model):
     __tablename__ = 'assoc_post_author'
 
-    post_id = db.Column(db.Integer, db.ForeignKey("posts.id"), nullable=False, primary_key=True)
-    user_id = db.Column(db.Integer, db.ForeignKey('users.id'), nullable=False, primary_key=True)
+    post_id = db.Column(db.Integer, db.ForeignKey(
+        'posts.id'), nullable=False, primary_key=True)
+    user_id = db.Column(db.Integer, db.ForeignKey(
+        'users.id'), nullable=False, primary_key=True)
     order = db.Column(db.Integer)
 
     post = db.relationship('Post', lazy='joined')
@@ -101,7 +106,8 @@ class Comment(db.Model):
     text = db.Column(db.Text)
     type = db.Column(db.String(100), default='post')
     created_at = db.Column(db.DateTime, default=func.now())
-    updated_at = db.Column(db.DateTime, default=func.now(), onupdate=func.now())
+    updated_at = db.Column(
+        db.DateTime, default=func.now(), onupdate=func.now())
 
 
 class ErrorLog(db.Model):
@@ -118,13 +124,16 @@ class ErrorLog(db.Model):
     @classmethod
     def from_exception(cls, e):
         tb = sys.exc_info()[-1]
-        filename, linenumber, function, code = traceback.extract_tb(sys.exc_info()[-1])[-1]
-        filename = os.path.relpath(filename, os.path.join(os.path.dirname(__file__), '..'))
+        filename, linenumber, function, code = traceback.extract_tb(
+            sys.exc_info()[-1])[-1]
+        filename = os.path.relpath(
+            filename, os.path.join(os.path.dirname(__file__), '..'))
+        e_args = '; '.join(str(a) for a in e.args)
         return ErrorLog(
             function=function,
-            location='{}:{}'.format(filename, linenumber),
-            message='{}: {}'.format(e.__class__.__name__, "; ".join(str(a) for a in e.args)),
-            traceback="\n".join(traceback.format_tb(tb))
+            location=f'{filename}:{linenumber}',
+            message=f'{e.__class__.__name__}: {e_args}',
+            traceback='\n'.join(traceback.format_tb(tb))
         )
 
     @classmethod
@@ -136,8 +145,7 @@ class ErrorLog(db.Model):
                 db_session.rollback()
                 db_session.add(ErrorLog.from_exception(e))
                 db_session.commit()
-                tb = sys.exc_info()[2]
-                raise e.with_traceback(tb=sys.exc_info()[2])
+                raise e.with_traceback(sys.exc_info()[2])
         return wrapped
 
 
@@ -156,7 +164,8 @@ class PageView(db.Model):
     created_at = db.Column(db.DateTime, default=func.now())
     version = db.Column(db.String(100), default=__version__)
 
-    __table_args__ = (Index("object_id_type_action_index", object_id, object_type, object_action),)
+    __table_args__ = (Index('object_id_type_action_index',
+                      object_id, object_type, object_action),)
 
     class logged(object):
 
@@ -179,8 +188,10 @@ class PageView(db.Model):
                 version=__version__
             )
             errorlog = None
-            log.object_id, log.object_type, log.object_action, reextract_after_request = self.extract_objects(*args, **kwargs)
-            db_session.add(log)  # Add log here to ensure pageviews are accurate
+            log.object_id, log.object_type, log.object_action, reextract_after_request = self.extract_objects(
+                *args, **kwargs)
+            # Add log here to ensure pageviews are accurate
+            db_session.add(log)
 
             try:
                 return self._route(*args, **kwargs)
@@ -190,13 +201,13 @@ class PageView(db.Model):
                 errorlog = ErrorLog.from_exception(e)
                 db_session.add(errorlog)
                 db_session.commit()
-                tb = sys.exc_info()[2]
-                raise e.with_traceback(tb=sys.exc_info()[2])
+                raise e.with_traceback(sys.exc_info()[2])
             finally:
                 # Extract object id and type after response generated (if requested) to ensure
                 # most recent data is collected
                 if reextract_after_request:
-                    log.object_id, log.object_type, log.object_action, _ = self.extract_objects(*args, **kwargs)
+                    log.object_id, log.object_type, log.object_action, _ = self.extract_objects(
+                        *args, **kwargs)
 
                 if errorlog is not None:
                     log.id_errorlog = errorlog.id
@@ -213,10 +224,14 @@ class PageView(db.Model):
             try:
                 object_info = self._object_extractor(*args, **kwargs)
             except Exception as e:
-                logger.warning("Error using object extractor: " + str(e))
+                logger.warning(f'Error using object extractor: {e}')
                 object_info = {'id': (-1), 'type': None}
-            assert isinstance(object_info, dict), "Object extractors must return a dictionary."
-            assert len(set(['id', 'type']).difference(object_info.keys())) == 0 and len(set(object_info.keys()).difference(['id', 'type', 'action', 'may_change'])) == 0, "Object extractors must at least include the keys 'id' and 'type', and optionally 'action' and 'may_change'. Was provided with: {}".format(str(list(object_info.keys())))
+            assert isinstance(
+                object_info, dict), 'Object extractors must return a dictionary.'
+            assert len(set(['id', 'type']).difference(object_info.keys())) == 0 and \
+                len(set(object_info.keys()).difference(['id', 'type', 'action', 'may_change'])) == 0, \
+                ("Object extractors must at least include the keys 'id' and 'type', and "
+                 f"optionally 'action' and 'may_change'. Was provided with: {list(object_info.keys())}")
             object_info = defaultdict(lambda: None, object_info)
             return object_info['id'], object_info['type'], object_info['action'], object_info['may_change'] or False
 
@@ -229,7 +244,8 @@ class Vote(db.Model):
     object_id = db.Column(db.Integer)
     object_type = db.Column(db.String(100), default='post')
     created_at = db.Column(db.DateTime, default=func.now())
-    updated_at = db.Column(db.DateTime, default=func.now(), onupdate=func.now())
+    updated_at = db.Column(
+        db.DateTime, default=func.now(), onupdate=func.now())
 
 
 @unique_constructor(
@@ -242,13 +258,16 @@ class User(db.Model, UserMixin):
     id = db.Column(db.Integer, primary_key=True)
     created_at = db.Column(db.DateTime, default=func.now())
 
-    identifier = db.Column(db.String(500))  # Unique identifier across all login methods
+    # Unique identifier across all login methods
+    identifier = db.Column(db.String(500))
 
-    username = db.Column(db.String(500))  # Username used to log in (may differ from identifier)
+    # Username used to log in (may differ from identifier)
+    username = db.Column(db.String(500))
     password = db.Column(db.String(500))  # Password for local logins
 
     name = db.Column(db.String(500))  # Name as determined by auth method
-    preferred_name = db.Column(db.String(500))  # Name as determined by user preferences
+    # Name as determined by user preferences
+    preferred_name = db.Column(db.String(500))
 
     email = db.Column(db.String(500))  # Email address
     avatar_uri = db.Column(db.Text())  # Either external url or data uri
@@ -256,8 +275,9 @@ class User(db.Model, UserMixin):
 
     last_login_at = db.Column(db.DateTime)  # Date of last login
 
-    _posts_assoc = db.relationship("PostAuthorAssoc")
-    posts = association_proxy('_posts_assoc', 'post')  # This property should not directly modified
+    _posts_assoc = db.relationship('PostAuthorAssoc')
+    # This property should not directly modified
+    posts = association_proxy('_posts_assoc', 'post')
 
     # Method overrides for the UserMixin class for flask_login
     @property
@@ -340,7 +360,7 @@ class Tag(db.Model):
     def description(self):
         if self._description:
             return self._description
-        return "All posts with tag '{}'.".format(self.name)
+        return f"All posts with tag '{self.name}'."
 
     @description.expression
     def description(self):
@@ -378,10 +398,10 @@ class Post(db.Model):
     created_at = db.Column(db.DateTime, default=func.now())
     updated_at = db.Column(db.DateTime, default=func.now())
 
-    _authors_assoc = db.relationship("PostAuthorAssoc",
+    _authors_assoc = db.relationship('PostAuthorAssoc',
                                      order_by='PostAuthorAssoc.order',
                                      collection_class=ordering_list('order'),
-                                     cascade="all, delete-orphan")
+                                     cascade='all, delete-orphan')
     _authors = association_proxy('_authors_assoc', 'author',
                                  creator=lambda author: PostAuthorAssoc(author=author),)
 
@@ -399,8 +419,7 @@ class Post(db.Model):
 
         for author in authors:
             if not isinstance(author, User):
-                author = author.strip()
-                author = User(identifier=author)
+                author = User(identifier=author.strip())
             user_objs.append(author)
 
         self._authors = user_objs
@@ -413,7 +432,7 @@ class Post(db.Model):
     def authors_string(self):
         raise NotImplementedError
 
-    _tags = db.relationship("Tag", secondary=assoc_post_tag, backref='posts',
+    _tags = db.relationship('Tag', secondary=assoc_post_tag, backref='posts',
                             lazy='subquery')
 
     @hybrid_property
@@ -431,7 +450,7 @@ class Post(db.Model):
         for tag in tags:
             if not isinstance(tag, Tag):
                 tag = tag.strip()
-                if tag[0] == "#":
+                if tag[0] == '#':
                     tag = tag[1:]
                 tag = Tag(name=tag)
             tag_objs.append(tag)
@@ -443,7 +462,7 @@ class Post(db.Model):
         excluded_tags = current_app.config.get('EXCLUDED_TAGS', [])
         return any([tag.name in excluded_tags for tag in self.tags])
 
-    _groups = db.relationship("Group", secondary=assoc_post_group, backref='posts',
+    _groups = db.relationship('Group', secondary=assoc_post_group, backref='posts',
                               lazy='subquery')
 
     @hybrid_property
@@ -462,7 +481,7 @@ class Post(db.Model):
 
         # create an implicit group, group_post.id, to add
         # single users to
-        group = Group(name=":post_group_" + str(self.id))
+        group = Group(name=f':post_group_{self.id}')
 
         # this created group should have the author associated with it
         # so they can add people to the post
@@ -486,7 +505,8 @@ class Post(db.Model):
         if status is None:
             self._status = None
         else:
-            assert isinstance(status, KnowledgeRepository.PostStatus), "Status must be an instance of KnowledgeRepository.PostStatus.Status or None"
+            assert isinstance(status, KnowledgeRepository.PostStatus), \
+                'Status must be an instance of KnowledgeRepository.PostStatus.Status or None'
             self._status = status.value
 
     @hybrid_property
@@ -497,7 +517,7 @@ class Post(db.Model):
     def is_published(self):
         return func.coalesce(self._status, 0) == current_repo.PostStatus.PUBLISHED.value
 
-    _views = db.relationship("PageView", lazy='dynamic',
+    _views = db.relationship('PageView', lazy='dynamic',
                              primaryjoin="and_(foreign(PageView.object_id)==Post.id, "
                                          "PageView.object_type=='post',"
                                          "PageView.object_action=='view')")
@@ -515,7 +535,7 @@ class Post(db.Model):
         return (select([func.count(PageView.id)])
                 .where(PageView.object_id == self.id)
                 .where(PageView.object_type == 'post')
-                .label("view_count"))
+                .label('view_count'))
 
     @hybrid_property
     def view_user_count(self):
@@ -529,9 +549,9 @@ class Post(db.Model):
         return (select([func.count(distinct(PageView.user_id))])
                 .where(PageView.object_id == self.id)
                 .where(PageView.object_type == 'post')
-                .label("view_user_count"))
+                .label('view_user_count'))
 
-    _votes = db.relationship("Vote", lazy='dynamic',
+    _votes = db.relationship('Vote', lazy='dynamic',
                              primaryjoin="and_(foreign(Vote.object_id)==Post.id, "
                                          "Vote.object_type=='post')")
 
@@ -549,14 +569,14 @@ class Post(db.Model):
         return (select([func.count(Vote.id)])
                 .where(Vote.object_id == self.id)
                 .where(Vote.object_type == 'post')
-                .label("vote_count"))
+                .label('vote_count'))
 
     def vote_counted_for_user(self, user_id):
         return (db_session.query(Vote)
                           .filter(and_(Vote.object_id == self.id, Vote.object_type == 'post', Vote.user_id == user_id))
                           .first()) is not None
 
-    _comments = db.relationship("Comment", lazy="dynamic",
+    _comments = db.relationship('Comment', lazy='dynamic',
                                 primaryjoin="and_(foreign(Comment.post_id)==Post.id, "
                                             "Comment.type=='post')")
 
@@ -574,7 +594,7 @@ class Post(db.Model):
         return (select([func.count(Comment.id)])
                 .where(Comment.post_id == self.id)
                 .where(Comment.object_type == 'post')
-                .label("comments_count"))
+                .label('comments_count'))
 
     @property
     def kp(self):
@@ -615,10 +635,10 @@ class Post(db.Model):
 
         self.status = kp.status
 
-        self.private = 0
+        self.private = False
         # we do this check so that no header (None) and False are treated the same
-        if headers.get('private', ''):
-            self.private = 1
+        if headers.get('private', False):
+            self.private = True
             self.groups = headers.get('allowed_groups', [])
 
 
@@ -648,13 +668,30 @@ class Group(db.Model):
     id = db.Column(db.Integer, primary_key=True)
     name = db.Column(db.String(128), unique=True)
 
-    _users = db.relationship("User", secondary=assoc_group_user, backref='users',
+    _users = db.relationship('User', secondary=assoc_group_user, backref='users',
                              lazy='subquery')
 
+    def _prepare_users(self, users):
+        user_objs = []
+
+        for user in users:
+            if not isinstance(user, User):
+                user = User(username=user.strip())
+            user_objs.append(user)
+
+        return user_objs
+
     @hybrid_property
     def users(self):
         return self._users
 
     @users.setter
-    def users(self, user_objs):
-        self._users = self._users + user_objs
+    def users(self, users):
+        self._users = self._prepare_users(users)
+
+    def users_add(self, users):
+        self._users += self._prepare_users(users)
+
+    def users_remove(self, users):
+        self._users = list(set(self._users).difference(
+            self._prepare_users(users)))
