@@ -1,9 +1,11 @@
+from . import auth
 from . import health
 from . import comment
 from . import index
-from . import render
+from . import posts
 from . import stats
 from . import tags
 from . import vote
-from . import web_editor
+from . import editor
 from . import groups
+from . import debug
