@@ -1,36 +1,55 @@
+from .. import permissions
+from ..index import update_index
+from ..models import Comment, PageView, Post, PostAuthorAssoc
+from ..proxies import current_repo, current_user, db_session
+from ..utils.emails import (
+    send_review_email,
+    send_reviewer_request_email,
+)
+from ..utils.image import (
+    is_allowed_image_format,
+    is_pdf,
+    pdf_page_to_png,
+)
+from ..utils.shared import get_blueprint
+from datetime import datetime
+from flask import (
+    current_app,
+    render_template,
+    request,
+    send_from_directory,
+    url_for,
+)
+from knowledge_repo.post import KnowledgePost
+from sqlalchemy import or_
+from urllib.parse import unquote
+from werkzeug.utils import secure_filename
 import json
 import logging
-import sys
 import os
-from datetime import datetime
-from urllib.parse import unquote
-
-from flask import request, render_template, Blueprint, current_app, url_for, send_from_directory, g
-from sqlalchemy import or_
-from werkzeug.utils import secure_filename
-
-from knowledge_repo.post import KnowledgePost
-from .. import permissions
-from ..proxies import db_session, current_repo, current_user
-from ..models import Post, PostAuthorAssoc, Tag, Comment, User, PageView
-from ..utils.emails import send_review_email, send_reviewer_request_email
-from ..utils.image import pdf_page_to_png, is_pdf, is_allowed_image_format
-
-from ..index import update_index
-
 
 logging.basicConfig(level=logging.INFO)
 logger = logging.getLogger(__name__)
 
-
-blueprint = Blueprint('editor', __name__,
-                      template_folder='../templates', static_folder='../static')
-
-# TODO: These functions have not been fully married to the KnowledgePost API
-# Currently, backended by Post objects but partially implemented on KnowledgePost API
+blueprint = get_blueprint('editor', __name__)
 
 
-# TODO: Deprecate this route in favour of integrating editing links into primary index pages and user pages
+def get_warning_msg(msg):
+    return json.dumps({'msg': msg, 'success': False})
+
+
+def get_error_msg(msg):
+    return json.dumps({'error_msg': msg, 'success': False})
+
+
+# TODO: These functions have not been fully married
+# to the KnowledgePost API
+# Currently, backended by Post objects but partially
+# implemented on KnowledgePost API
+
+
+# TODO: Deprecate this route in favour of integrating editing
+# links into primary index pages and user pages
 @blueprint.route('/webposts', methods=['GET'])
 @PageView.logged
 @permissions.post_edit.require()
@@ -40,17 +59,18 @@ def gitless_drafts():
     """
     prefixes = current_app.config.get('WEB_EDITOR_PREFIXES', [])
     if prefixes == []:
-        raise Exception("Web editing is not configured")
+        raise Exception('Web editing is not configured')
 
     query = (db_session.query(Post))
     if prefixes is not None:
         query = query.filter(or_(*[Post.path.like(p + '%') for p in prefixes]))
 
     if current_user.identifier not in current_repo.config.editors:
-        query = (query.outerjoin(PostAuthorAssoc, PostAuthorAssoc.post_id == Post.id)
-                      .filter(PostAuthorAssoc.user_id == current_user.id))
+        query = (query.outerjoin(
+            PostAuthorAssoc, PostAuthorAssoc.post_id == Post.id)
+            .filter(PostAuthorAssoc.user_id == current_user.id))
 
-    return render_template("web_posts.html", posts=query.all())
+    return render_template('web_posts.html', posts=query.all())
 
 
 @blueprint.route('/edit')
@@ -66,7 +86,7 @@ def editor(path=None):
     if prefixes is not None:
         assert (
             path is None or any(path.startswith(prefix) for prefix in prefixes)
-        ), "Editing of this post online is not permitted by server configuration."
+        ), 'Editing this post online is not permitted by server configuration.'
 
     # set defaults
     data = {'title': None,
@@ -95,10 +115,11 @@ def editor(path=None):
         if post:  # post may have not been indexed yet
             data['comments'] = (db_session.query(Comment)
                                           .filter(Comment.post_id == post.id)
-                                          .filter(Comment.type == "review")
+                                          .filter(Comment.type == 'review')
                                           .all())
 
-    if current_user.identifier not in data['authors'] or current_user.identifier in current_repo.config.editors:
+    if current_user.identifier not in data['authors'] \
+            or current_user.identifier in current_repo.config.editors:
         data['can_approve'] = 1
 
     data['created_at'] = data['created_at']
@@ -123,32 +144,35 @@ def save_post():
 
     prefixes = current_app.config['WEB_EDITOR_PREFIXES']
     if prefixes == []:
-        raise Exception("Web editing is not configured")
+        raise Exception('Web editing is not configured')
 
     if prefixes is not None:
         if not any([path.startswith(prefix) for prefix in prefixes]):
-            return json.dumps({'msg': ("Your post path must begin with one of {}").format(prefixes),
-                               'success': False})
+            return get_warning_msg(
+                f'Your post path must begin with one of {prefixes}')
 
     # TODO better handling of overwriting
     kp = None
     if path in current_repo:
         kp = current_repo.post(path)
-        if current_user.identifier not in kp.headers['authors'] and current_user.identifier not in current_repo.config.editors:
-            return json.dumps({'msg': ("Post with path {} already exists and you are not an author!"
-                                       "\nPlease try a different path").format(path),
-                               'success': False})
+        if current_user.identifier not in kp.headers['authors'] \
+                and current_user.identifier not in current_repo.config.editors:
+            return get_warning_msg(
+                f'Post with path {path} already exists and you are not '
+                'an author!\nPlease try a different path')
 
     # create the knowledge post
     kp = kp or KnowledgePost(path=path)
 
     headers = {}
-    headers['created_at'] = datetime.strptime(data['created_at'], '%Y-%m-%d').date()
-    headers['updated_at'] = datetime.strptime(data['updated_at'], '%Y-%m-%d').date()
+    headers['created_at'] = datetime.strptime(
+        data['created_at'], '%Y-%m-%d').date()
+    headers['updated_at'] = datetime.strptime(
+        data['updated_at'], '%Y-%m-%d').date()
     headers['title'] = data['title']
     headers['path'] = data['path']
-    # TODO: thumbnail header not working currently, as feed image set with kp
-    # method not based on header
+    # TODO: thumbnail header not working currently, as feed image set
+    # with kp method not based on header
     headers['thumbnail'] = data.get('feed_image', '')
     headers['authors'] = [auth.strip() for auth in data['author']]
     headers['tldr'] = data['tldr']
@@ -158,7 +182,8 @@ def save_post():
 
     kp.write(unquote(data['markdown']), headers=headers)
     # add to repo
-    current_repo.add(kp, update=True, message=headers['title'])  # THIS IS DANGEROUS
+    current_repo.add(
+        kp, update=True, message=headers['title'])  # THIS IS DANGEROUS
 
     update_index()
     return json.dumps({'path': path})
@@ -190,7 +215,7 @@ def publish_post():
     """ Publish the post by changing the status """
     path = request.args.get('path', None)
     if path not in current_repo:
-        return json.dumps({'msg': "Unable to retrieve post with path = {}!".format(path), 'success': False})
+        return get_warning_msg(f'Unable to retrieve post with path = {path}!')
     current_repo.publish(path)
 
     update_index(check_timeouts=False)
@@ -204,7 +229,7 @@ def unpublish_post():
     """ Unpublish the post """
     path = request.args.get('path', None)
     if path not in current_repo:
-        return json.dumps({'msg': "Unable to retrieve post with path = {}!".format(path), 'success': False})
+        return get_warning_msg(f'Unable to retrieve post with path = {path}!')
     current_repo.unpublish(path)
 
     update_index(check_timeouts=False)
@@ -218,7 +243,7 @@ def accept():
     """ Accept the post """
     path = request.args.get('path', None)
     if path not in current_repo:
-        return json.dumps({'msg': "Unable to retrieve post with path = {}!".format(path), 'success': False})
+        return get_warning_msg(f'Unable to retrieve post with path = {path}!')
     current_repo.accept(path)
     update_index()
     return 'OK'
@@ -231,10 +256,11 @@ def delete_post():
     """ Delete a post """
     path = request.args.get('path', None)
     if path not in current_repo:
-        return json.dumps({'msg': "Unable to retrieve post with path = {}!".format(path), 'success': False})
+        return get_warning_msg(f'Unable to retrieve post with path = {path}!')
     kp = current_repo.post(path)
     if current_user.identifier not in kp.headers['authors']:
-        return json.dumps({'msg': "You can only delete a post where you are an author!", 'success': False})
+        return get_warning_msg(
+            'You can only delete a post where you are an author!')
     current_repo.remove(path)
 
     update_index(check_timeouts=False)
@@ -246,7 +272,8 @@ def delete_post():
 @permissions.post_edit.require()
 def review_comment():
     """
-    Saves a review and sends an email that the post has been reviewed to the author of the post or deletes a submitted review
+    Saves a review and sends an email that the post has been reviewed to
+    the author of the post or deletes a submitted review
     """
 
     if request.method == 'POST':
@@ -257,7 +284,7 @@ def review_comment():
         comment.text = request.get_json()['text']
         comment.user_id = current_user.id
         comment.post_id = post_id
-        comment.type = "review"
+        comment.type = 'review'
         db_session.add(comment)
         db_session.commit()
 
@@ -279,8 +306,9 @@ def review_comment():
 @PageView.logged
 @permissions.post_edit.require()
 def file_upload():
-    """ Uploads images dropped on the web editor's markdown box to static/images
-        and notifies editors by email
+    """
+    Uploads images dropped on the web editor's markdown box to
+    static/images and notifies editors by email
     """
     upload_folder = 'images'
     title = request.form['title']
@@ -289,18 +317,19 @@ def file_upload():
 
     if files:
         for img_file in files.values():
-            filename = secure_filename(title + "_" + img_file.filename).lower()
+            filename = secure_filename(title + '_' + img_file.filename).lower()
             dst_folder = os.path.join(current_app.static_folder, upload_folder)
 
             if is_allowed_image_format(img_file):
                 try:
                     img_file.save(os.path.join(dst_folder, filename))
                     send_from_directory(dst_folder, filename)
-                    uploadedFiles += [url_for("static", filename=os.path.join(upload_folder, filename))]
+                    uploadedFiles += [url_for('static', filename=os.path.join(
+                        upload_folder, filename))]
                 except Exception as e:
-                    error_msg = "ERROR during image upload: {}".format(str(e))
+                    error_msg = f'ERROR during image upload: {e}'
                     logger.error(error_msg)
-                    return json.dumps({'error_msg': error_msg, 'success': False})
+                    return get_error_msg(error_msg)
 
             elif is_pdf(filename):
                 from PyPDF2 import PdfFileReader
@@ -310,12 +339,16 @@ def file_upload():
                     num_pages = src_pdf.getNumPages()
                     for page_num in range(num_pages):
                         page_png = pdf_page_to_png(src_pdf, page_num)
-                        page_name = "{filename}_{page_num}.jpg".format(**locals())
-                        page_png.save(filename=os.path.join(dst_folder, page_name))
-                        uploadedFiles += [url_for("static", filename=os.path.join(upload_folder, page_name))]
+                        page_name = '{filename}_{page_num}.jpg'.format(
+                            **locals())
+                        page_png.save(filename=os.path.join(
+                            dst_folder, page_name))
+                        uploadedFiles += [url_for(
+                            'static', filename=os.path.join(
+                                upload_folder, page_name))]
                 except Exception as e:
-                    error_msg = "ERROR during pdf upload: {}".format(str(e))
+                    error_msg = f'ERROR during pdf upload: {e}'
                     logger.error(error_msg)
-                    return json.dumps({'error_msg': error_msg, 'success': False})
+                    return get_error_msg(error_msg)
 
     return json.dumps({'links': uploadedFiles, 'success': True})
