@@ -1,10 +1,10 @@
+from knowledge_repo._version import __optional_dependencies__
+from knowledge_repo.constants import PDF
+from knowledge_repo.utils.dependencies import check_dependencies
 import imghdr
 import io
 import os
 
-from knowledge_repo.utils.dependencies import check_dependencies
-from knowledge_repo._version import __optional_dependencies__
-
 ALLOWED_IMAGE_TYPES = ('png', 'jpeg', 'gif')
 
 
@@ -29,10 +29,14 @@ def pdf_page_to_png(src_pdf, pagenum=0, resolution=154):
     :param int resolution: Resolution for resulting png in DPI.
     """
 
-    check_dependencies(__optional_dependencies__['pdf'])
-    # Import libraries within this function so as to avoid import-time dependence
+    check_dependencies(__optional_dependencies__[PDF])
+    # Import libraries within this function so as to avoid
+    # import-time dependence
     import PyPDF2
-    from wand.image import Image  # TODO: When we start using this again, document which system-level libraries are required.
+
+    # TODO: When we start using this again, document which
+    # system-level libraries are required.
+    from wand.image import Image
 
     dst_pdf = PyPDF2.PdfFileWriter()
     dst_pdf.addPage(src_pdf.getPage(pagenum))
