@@ -1,7 +1,3 @@
-
-
-from __future__ import absolute_import
-from __future__ import unicode_literals
 from markdown import Extension
 from markdown.preprocessors import Preprocessor
 
