@@ -1,11 +1,13 @@
+from .constants import PY_EXTENSION, YML_EXTENSION
+from .utils.files import read_yaml
+
 import functools
 import importlib
-import sys
 import logging
 import os
+import sys
 import time
 import types
-import yaml
 
 logger = logging.getLogger(__name__)
 
@@ -33,7 +35,8 @@ class KnowledgeRepositoryConfig(dict):
         self[attr] = value
 
     def __dir__(self):
-        return list(set(list(self.DEFAULT_CONFIGURATION.keys()) + list(self.keys())))
+        return list(set(
+            list(self.DEFAULT_CONFIGURATION.keys()) + list(self.keys())))
 
     def update(self, *values, **kwargs):
         for value in values:
@@ -49,11 +52,11 @@ class KnowledgeRepositoryConfig(dict):
                     self.__update_from_file(value)
                 else:
                     logger.warning(
-                        "Configuration file {} does not exist.".format(value))
+                        f'Configuration file {value} does not exist.')
             elif isinstance(value, type(None)):
                 pass
             else:
-                raise ValueError("Cannot interpret {}".format(value))
+                raise ValueError(f'Cannot interpret {value}')
         dict.update(self, kwargs)
 
     def update_defaults(self, *values, **kwargs):
@@ -66,11 +69,12 @@ class KnowledgeRepositoryConfig(dict):
                 if os.path.exists(value):
                     self.__defaults_from_file(value)
                 else:
-                    logger.warning("Configuration file {} does not exist.".format(value))
+                    logger.warning(
+                        f'Configuration file {value} does not exist.')
             elif isinstance(value, type(None)):
                 pass
             else:
-                raise ValueError("Cannot interpret {}".format(value))
+                raise ValueError(f'Cannot interpret {value}')
         self.DEFAULT_CONFIGURATION.update(kwargs)
 
     def __defaults_from_file(self, filename):
@@ -86,16 +90,17 @@ class KnowledgeRepositoryConfig(dict):
         self.__set_from_module(self, module)
 
     def __set_from_file(self, d, filename, force=False):
-        if filename.endswith('.py'):
-            module_name = 'knowledge_repo.config_{}'.format(str(time.time()).replace('.', ''))
-            spec = importlib.util.spec_from_file_location(module_name, filename)
+        if filename.endswith(PY_EXTENSION):
+            time_str = str(time.time()).replace('.', '')
+            module_name = f'knowledge_repo.config_{time_str}'
+            spec = importlib.util.spec_from_file_location(
+                module_name, filename)
             config = importlib.util.module_from_spec(spec)
             sys.modules[module_name] = module
             spec.loader.exec_module(module)
             self.__set_from_module(d, config, force)
-        elif filename.endswith('.yml'):
-            with open(filename) as f:
-                config = yaml.safe_load(f)
+        elif filename.endswith(YML_EXTENSION):
+            config = read_yaml(filename)
             self.update(config)
 
     def __set_from_module(self, d, module, force=False):
@@ -103,6 +108,7 @@ class KnowledgeRepositoryConfig(dict):
             if not key.startswith('_'):
                 if not force and key not in self.DEFAULT_CONFIGURATION:
                     logger.debug(
-                        "Ignoring configuration key `{}` which is not a valid configuration key.".format(key))
+                        f'Ignoring configuration key `{key}` '
+                        'which is not a valid configuration key.')
                 else:
                     d[key] = getattr(module, key)
