@@ -1,24 +1,29 @@
-
-import os
-from functools import wraps
-
+from .constants import (
+    EXTRACT_IMAGES,
+    FORMAT_CHECKS,
+    GDOC,
+    PROXY,
+)
 from .postprocessor import KnowledgePostProcessor
-from .utils.registry import SubclassRegisteringABCMeta
 from .utils.dependencies import check_dependencies
+from .utils.registry import SubclassRegisteringABCMeta
+from functools import wraps
+import os
 
 
 def get_format(filename, format=None):
     if format is None:
         if filename.startswith('https://docs.google.com/document/d/'):
-            format = 'gdoc'
+            format = GDOC
         elif filename.startswith('http://') or filename.startswith('https://'):
-            format = 'proxy'
+            format = PROXY
         elif '.' in filename:
             format = os.path.splitext(filename)[1]
             if format.startswith('.'):
                 format = format[1:]
         else:
-            raise RuntimeError("Unable to determine a format automatically. Please manually specify the format, and try again.")
+            raise RuntimeError(
+                "Unable to determine a format automatically. Please manually specify the format, and try again.")
     return format
 
 
@@ -26,11 +31,14 @@ class KnowledgePostConverter(object, metaclass=SubclassRegisteringABCMeta):
     _registry_keys = None  # File extensions
 
     def __init__(self, kp, format=None, postprocessors=None, interactive=False, **kwargs):
-        check_dependencies(self.dependencies, "Whoops! You are missing some dependencies required to use `{}` instances.".format(self.__class__.__name__))
+        check_dependencies(
+            self.dependencies,
+            f'Whoops! You are missing some dependencies required to use `{self.__class__.__name__}` instances.',
+        )
         self.kp = kp
         self.format = format
         if postprocessors is None:
-            postprocessors = [('extract_images', {}), ('format_checks', {})]
+            postprocessors = [(EXTRACT_IMAGES, {}), (FORMAT_CHECKS, {})]
         self.postprocessors = postprocessors
         self.interactive = interactive
         self.init(**kwargs)
@@ -57,11 +65,13 @@ class KnowledgePostConverter(object, metaclass=SubclassRegisteringABCMeta):
                 if postprocessors is None:
                     postprocessors = []
                 for postprocessor, kwargs in postprocessors:
-                    KnowledgePostProcessor._get_subclass_for(postprocessor)(**kwargs).process(kp)
+                    KnowledgePostProcessor._get_subclass_for(
+                        postprocessor)(**kwargs).process(kp)
 
                 return kp
             finally:
                 self.cleanup()
+
         return wrapped
 
     def __getattribute__(self, attr):
@@ -94,6 +104,9 @@ class KnowledgePostConverter(object, metaclass=SubclassRegisteringABCMeta):
     @classmethod
     def for_format(cls, kp, format, postprocessors=None, interactive=False):
         if format.lower() not in cls._registry:
-            raise ValueError("The knowledge repository does not support files of type '{}'. Supported types are: {}."
-                             .format(format, ','.join(list(cls._registry.keys()))))
-        return cls._get_subclass_for(format.lower())(kp, format=format, postprocessors=postprocessors, interactive=interactive)
+            keys = ','.join(list(cls._registry.keys()))
+            raise ValueError(
+                f"The knowledge repository does not support files of type '{format}'. Supported types are: {keys}."
+            )
+        return cls._get_subclass_for(format.lower())(kp, format=format, postprocessors=postprocessors,
+                                                     interactive=interactive)
