@@ -1,18 +1,20 @@
+from ..constants import DOCX
+from ..converter import KnowledgePostConverter
+from knowledge_repo.utils.files import read_text
 import os
 import re
 import shutil
 import tempfile
 
-from ..converter import KnowledgePostConverter
-
 
 class DocxConverter(KnowledgePostConverter):
 
-    _registry_keys = ['docx']
+    _registry_keys = [DOCX]
 
     @property
     def dependencies(self):
-        # Dependencies required for this converter on top of core knowledge-repo dependencies
+        # Dependencies required for this converter on top of core
+        # knowledge-repo dependencies
         return ['pypandoc']
 
     def from_file(self, filename, **opts):
@@ -22,24 +24,25 @@ class DocxConverter(KnowledgePostConverter):
         import pypandoc
         pypandoc.convert_file(
             filename,
-            format='docx',
+            format=DOCX,
             to='markdown-grid_tables',
             outputfile=target_file,
             extra_args=[
                 '--standalone',
                 '--wrap=none',
-                '--extract-media={}'.format(wd)
+                f'--extract-media={wd}'
             ]
         )
 
-        with open(target_file) as f:
-            md = f.read()
+        md = read_text(target_file)
 
         # Image embeddings exported from docx files have fixed sizes in inches
         # which browsers do not understand. We remove these annotations.
-        md = re.sub(r'(\!\[[^\]]+?\]\([^\)]+?\))\{[^\}]+?\}', lambda m: m.group(1), md)
+        md = re.sub(
+            r'(\!\[[^\]]+?\]\([^\)]+?\))\{[^\}]+?\}', lambda m: m.group(1), md)
 
-        # Write markdown content to knowledge post (images will be extracted later)
+        # Write markdown content to knowledge post (images will be extracted
+        # later)
         self.kp_write(md)
 
     def cleanup(self):
