@@ -1,19 +1,16 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-import markdown
-from markdown import Extension
-from markdown.blockprocessors import BlockProcessor
-from markdown.preprocessors import Preprocessor
-from markdown.util import AtomicString, etree
-from markdown.extensions import codehilite, toc
-from markdown.inlinepatterns import Pattern
-
-import re
-import base64
-import mimetypes
-
+from __future__ import absolute_import, unicode_literals
+from ..constants import PROXY, UTF8
 from ..converter import KnowledgePostConverter
 from ..mapping import SubstitutionMapper
+from markdown import Extension
+from markdown.blockprocessors import BlockProcessor
+from markdown.extensions import codehilite, toc
+from markdown.inlinepatterns import Pattern
+from markdown.preprocessors import Preprocessor
+from markdown.util import AtomicString, etree
+import base64
+import markdown
+import mimetypes
 
 MARKDOWN_EXTENSIONS = ['extra',
                        'abbr',
@@ -115,19 +112,25 @@ class IndentsAsCellOutputProcessor(BlockProcessor):
             # new code blocks, append this block to the previous, adding back
             # linebreaks removed from the split into a list.
 
-            block_is_html = block_is_html and not isinstance(sibling.text, AtomicString)
+            block_is_html = block_is_html and not isinstance(
+                sibling.text, AtomicString)
 
             block = u'\n'.join([sibling.text, block])
             output = sibling
         else:
             # This is a new codeblock. Create the elements and insert text.
-            output = markdown.util.etree.SubElement(parent, 'div', {'class': 'code-output'})
+            output = markdown.util.etree.SubElement(
+                parent, 'div', {'class': 'code-output'})
 
-        # If not HTML, add the `pre` class so that we know to render output as raw text
-        if not block_is_html and 'pre' not in output.get('class', 'code-output'):
+        # If not HTML, add the `pre` class
+        # so that we know to render output as raw text
+        if not block_is_html and 'pre' not in output.get(
+            'class', 'code-output'
+        ):
             output.set('class', ' '.join([output.get('class', ''), 'pre']))
 
-        output.text = "{}\n".format(block) if block_is_html else AtomicString("{}\n".format(block))
+        output.text = f'{block}\n' if block_is_html else AtomicString(
+            f'{block}\n')
 
         if theRest:
             # This block contained unindented line(s) after the first indented
@@ -142,7 +145,8 @@ class IndentsAsCellOutput(Extension):
         md.preprocessors.add("code_isolation",
                              IndentsAsCellOutputPreprocessor(md),
                              "<html_block")
-        md.parser.blockprocessors['code'] = IndentsAsCellOutputProcessor(md.parser)
+        md.parser.blockprocessors['code'] = IndentsAsCellOutputProcessor(
+            md.parser)
 
 
 class KnowledgeMetaPreprocessor(Preprocessor):
@@ -172,17 +176,20 @@ class KnowledgeMetaExtension(Extension):
 class MathJaxPattern(markdown.inlinepatterns.Pattern):
 
     def __init__(self):
-        markdown.inlinepatterns.Pattern.__init__(self, r'(?<!\\)(\$\$?)(.+?)\2')
+        markdown.inlinepatterns.Pattern.__init__(
+            self, r'(?<!\\)(\$\$?)(.+?)\2')
 
     def handleMatch(self, m):
         node = markdown.util.etree.Element('mathjax')
-        node.text = markdown.util.AtomicString(m.group(2) + m.group(3) + m.group(2))
+        node.text = markdown.util.AtomicString(
+            m.group(2) + m.group(3) + m.group(2))
         return node
 
 
 class MathJaxExtension(markdown.Extension):
     def extendMarkdown(self, md, md_globals=None):
-        # Needs to come before escape matching because \ is pretty important in LaTeX
+        # Needs to come before escape matching because \ is
+        # pretty important in LaTeX
         md.inlinePatterns.add('mathjax', MathJaxPattern(), '<escape')
 
 
@@ -195,7 +202,10 @@ class HTMLConverter(KnowledgePostConverter):
     def init(self):
         self.kp_images = self.kp.read_images()
 
-    def _render_markdown(self, skip_headers=False, images_base64_encode=True, urlmappers=[]):
+    def _render_markdown(self,
+                         skip_headers=False,
+                         images_base64_encode=True,
+                         urlmappers=[]):
         """
         Returns the `Markdown` instance as well as the rendered html output
         as a tuple: (`Markdown`, str).
@@ -207,8 +217,11 @@ class HTMLConverter(KnowledgePostConverter):
             urlmappers.insert(0, self.base64_encode_image_mapper)
 
         # proxy posts are assumed to be embeddable links
-        if 'proxy' in self.kp.headers:
-            return None, '<a href="{0}">Linked Post</a>\n<iframe width=100% height=1000 src="{0}"></iframe>'.format(self.kp.headers['proxy'].strip())
+        if PROXY in self.kp.headers:
+            proxy = self.kp.headers[PROXY].strip()
+            return None, (f'<a href="{proxy}">Linked Post</a>\n'
+                          f'<iframe width=100% height=1000 '
+                          f'src="{proxy}"></iframe>')
 
         html = ''
         if not skip_headers:
@@ -221,7 +234,10 @@ class HTMLConverter(KnowledgePostConverter):
 
         return md, html
 
-    def to_string(self, skip_headers=False, images_base64_encode=True, urlmappers=[]):
+    def to_string(self,
+                  skip_headers=False,
+                  images_base64_encode=True,
+                  urlmappers=[]):
         """
         Renders the markdown as html
         """
@@ -244,7 +260,8 @@ class HTMLConverter(KnowledgePostConverter):
                     return match.group(0).replace(match.group('url'), new_url)
             return None
 
-        return SubstitutionMapper(patterns=patterns, mappers=[urlmapper_proxy]).apply(html)
+        return SubstitutionMapper(patterns=patterns,
+                                  mappers=[urlmapper_proxy]).apply(html)
 
     # Utility methods
     def render_headers(self):
@@ -275,5 +292,6 @@ class HTMLConverter(KnowledgePostConverter):
                 image_data = base64.b64encode(self.kp_images[url])
                 image_mimetype = mimetypes.guess_type(url)[0]
                 if image_mimetype is not None:
-                    return 'data:{};base64, '.format(image_mimetype) + image_data.decode('utf-8')
+                    return f'data:{image_mimetype};base64, ' + \
+                        image_data.decode(UTF8)
         return None
