@@ -1,14 +1,13 @@
-import os
-import shutil
-import random
-import string
-import logging
-import time
-import tempfile
-from urllib.parse import urljoin
-
 from .extract_images import ExtractImages
-
+from knowledge_repo.utils.files import write_binary
+from urllib.parse import urljoin
+import logging
+import os
+import random
+import shutil
+import string
+import tempfile
+import time
 
 logger = logging.getLogger(__name__)
 
@@ -16,14 +15,14 @@ logger = logging.getLogger(__name__)
 class ExtractImagesToLocalServer(ExtractImages):
     """
     This KnowledgePostProcessor subclass extracts images from posts to a local
-    directory. It is assumed that a local http server is then serving these images
-    from the local directory. It is designed to be used upon addition to a knowledge
-    repository, which can reduce the size of repositories. It replaces local images
-    with urls relative to `http_image_root` (the base url of the local http server).
-
-    `image_dir` should be the root of the image folder which is accessible locally.
-    `http_image_root` should be the root of the server where the images will be
-    accessible (e.g. 'http://localhost:8000').
+    directory. It is assumed that a local http server is then serving these
+    images from the local directory. It is designed to be used upon addition
+    to a knowledge repository, which can reduce the size of repositories. It
+    replaces local images with urls relative to `http_image_root` (the base
+    url of the local http server).
+    `image_dir` should be the root of the image folder which is accessible
+    locally. `http_image_root` should be the root of the server where the
+    images will be accessible (e.g. 'http://localhost:8000').
     """
 
     _registry_keys = ['extract_images_to_local']
@@ -36,8 +35,7 @@ class ExtractImagesToLocalServer(ExtractImages):
         # Copy image data to new file
         if is_ref:
             _, tmp_path = tempfile.mkstemp()
-            with open(tmp_path, 'wb') as f:
-                f.write(kp._read_ref(img_path))
+            write_binary(tmp_path, kp._read_ref(img_path))
         else:
             tmp_path = img_path
 
@@ -46,13 +44,11 @@ class ExtractImagesToLocalServer(ExtractImages):
             img_ext = os.path.splitext(img_path)[1]
 
             # Make random filename for image
-            random_string = ''.join(
+            random_name = ''.join(
                 random.choice(string.ascii_lowercase) for i in range(6))
-            fname_img = '{repo_name}_{time}_{random_string}{ext}'.format(
-                repo_name=repo_name,
-                time=int(round(time.time() * 100)),
-                random_string=random_string,
-                ext=img_ext).strip().replace(' ', '-')
+            timestamp = int(round(time.time() * 100))
+            fname_img = (f'{repo_name}_{timestamp}_'
+                         f'{random_name}{img_ext}').strip().replace(' ', '-')
 
             # See if a static file directory exists, if not, let's create
             if not os.path.exists(self.image_dir):
@@ -60,7 +56,7 @@ class ExtractImagesToLocalServer(ExtractImages):
 
             # Copy images to local http server directory
             new_path = os.path.join(self.image_dir, fname_img)
-            logger.info("Copying image {} to {}".format(tmp_path, new_path))
+            logger.info(f'Copying image {tmp_path} to {new_path}')
             try:
                 shutil.copyfile(tmp_path, new_path)
             except Exception as e:
