@@ -1,12 +1,12 @@
-import os
-import shutil
-import logging
-import time
-from io import open
-
 from ..post import KnowledgePost
 from ..repository import KnowledgeRepository
 from ..utils.encoding import encode
+from ..utils.files import get_path
+from knowledge_repo.utils.files import read_binary, write_binary
+import logging
+import os
+import shutil
+import time
 
 logger = logging.getLogger(__name__)
 
@@ -15,8 +15,10 @@ class FolderKnowledgeRepository(KnowledgeRepository):
     _registry_keys = ['', 'file']
 
     TEMPLATES = {
-        'README.md': os.path.abspath(os.path.join(os.path.dirname(__file__), '../templates', 'repository_readme.md')),
-        '.knowledge_repo_config.yml': os.path.abspath(os.path.join(os.path.dirname(__file__), '../templates', 'repository_config.yml'))
+        'README.md': get_path(
+            __file__, '../templates', 'repository_readme.md'),
+        '.knowledge_repo_config.yml': get_path(
+            __file__, '../templates', 'repository_config.yml'),
     }
 
     @classmethod
@@ -33,15 +35,15 @@ class FolderKnowledgeRepository(KnowledgeRepository):
             if not os.path.exists(target):
                 shutil.copy(template, target)
             else:
-                logger.warning("Not overriding existing file '{}'.".format(filename))
+                logger.warning(f"Not overriding existing file '{filename}'.")
         return FolderKnowledgeRepository(path)
 
     @classmethod
     def from_uri(cls, uri, *args, **kwargs):
         """
         If this folder is actually a git repository, a `GitKnowledgeRepository`
-        is returned instead, unless the folder knowledge repository is explicitly
-        requested via the 'file://' protocol.
+        is returned instead, unless the folder knowledge repository is
+        explicitly requested via the 'file://' protocol.
         """
         check_for_git = True
         if uri.startswith('file://'):
@@ -63,17 +65,17 @@ class FolderKnowledgeRepository(KnowledgeRepository):
 
     @path.setter
     def path(self, path):
-        assert isinstance(path, str), "The path specified must be a string."
+        assert isinstance(path, str), 'The path specified must be a string.'
         path = os.path.abspath(os.path.expanduser(path))
         if not os.path.exists(path):
             path = os.path.abspath(path)
             if self.auto_create:
                 self.create(path)
             else:
-                raise ValueError("Provided path '{}' does not exist.".format(path))
+                raise ValueError(f"Provided path '{path}' does not exist.")
         self._path = path
 
-    # ----------- Repository actions / state ------------------------------------
+    # ----------- Repository actions / state ----------------------------------
     @property
     def revision(self):
         return time.time()
@@ -93,7 +95,8 @@ class FolderKnowledgeRepository(KnowledgeRepository):
 
         if self.PostStatus.PUBLISHED in statuses:
 
-            for path, folders, files in os.walk(os.path.join(self.path, prefix or '')):
+            for path, folders, files in os.walk(
+                    os.path.join(self.path, prefix or '')):
 
                 # Do not visit hidden folders
                 for folder in folders:
@@ -101,7 +104,8 @@ class FolderKnowledgeRepository(KnowledgeRepository):
                         folders.remove(folder)
 
                 posts.update(
-                    os.path.join(os.path.relpath(path, start=self.path), folder)
+                    os.path.join(os.path.relpath(
+                        path, start=self.path), folder)
                     for folder in folders if folder.endswith('.kp')
                 )
                 posts.update(
@@ -109,7 +113,8 @@ class FolderKnowledgeRepository(KnowledgeRepository):
                     for file in files if file.endswith('.kp')
                 )
 
-        for post in sorted([post[2:] if post.startswith('./') else post for post in posts]):
+        for post in sorted([post[2:] if post.startswith('./')
+                            else post for post in posts]):
             yield post
 
     # ------------- Post submission / addition user flow ----------------------
@@ -134,16 +139,18 @@ class FolderKnowledgeRepository(KnowledgeRepository):
     def _remove(self, path, all=False):
         shutil.rmtree(os.path.join(self.path, path))
 
-    # ------------ Knowledge Post Data Retrieval Methods -------------------------
+    # ------------ Knowledge Post Data Retrieval Methods ----------------------
 
     def _kp_uuid(self, path):
         try:
             return self._kp_read_ref(path, 'UUID')
-        except:
+        except Exception as e:
+            print(f'Exception encountered: {e}')
             return None
 
     def _kp_path(self, path, rel=None):
-        return KnowledgeRepository._kp_path(self, os.path.expanduser(path), rel=rel or self.path)
+        return KnowledgeRepository._kp_path(
+            self, os.path.expanduser(path), rel=rel or self.path)
 
     def _kp_exists(self, path, revision=None):
         return os.path.exists(os.path.join(self.path, path))
@@ -152,11 +159,12 @@ class FolderKnowledgeRepository(KnowledgeRepository):
         return self.PostStatus.PUBLISHED
 
     def _kp_get_revision(self, path):
-        # We use a 'REVISION' file in the knowledge post folder rather than using git
-        # revisions because using git rev-parse is slow.
+        # We use a 'REVISION' file in the knowledge post folder rather than
+        # using git revisions because using git rev-parse is slow.
         try:
             return int(self._kp_read_ref(path, 'REVISION'))
-        except:
+        except Exception as e:
+            print(f'Exception encountered: {e}')
             return 0
 
     def _kp_get_revisions(self, path):
@@ -173,25 +181,28 @@ class FolderKnowledgeRepository(KnowledgeRepository):
             ref_dir = os.path.dirname(ref_path)
             if not os.path.exists(ref_dir):
                 os.makedirs(ref_dir)
-            with open(ref_path, 'wb') as f:
-                return f.write(data)
+            write_binary(ref_path, data)
 
-    def _kp_dir(self, path, parent=None, revision=None):  # TODO: Account for revision
+    # TODO: Account for revision
+    def _kp_dir(self, path, parent=None, revision=None):
         path = os.path.join(self.path, path)
         if os.path.isdir(path):
             if parent:
                 path = os.path.join(path, parent)
-            for dirpath, dirnames, filenames in os.walk(os.path.join(self.path, path)):
+            for dirpath, dirnames, filenames in os.walk(
+                    os.path.join(self.path, path)):
                 for filename in filenames:
-                    if dirpath == "" and filename == "REVISION":
+                    if dirpath == '' and filename == 'REVISION':
                         continue
-                    yield os.path.relpath(os.path.join(dirpath, filename), os.path.join(self.path, path))
+                    yield os.path.relpath(os.path.join(
+                        dirpath, filename), os.path.join(self.path, path))
         else:
             kp = KnowledgePost.from_file(path, format='kp')
             for reference in kp._dir(parent=parent):
                 yield reference
 
-    def _kp_has_ref(self, path, reference, revision=None):  # TODO: Account for revision
+    # TODO: Account for revision
+    def _kp_has_ref(self, path, reference, revision=None):
         path = os.path.join(self.path, path)
         if os.path.isdir(path):
             return os.path.isfile(os.path.join(path, reference))
@@ -203,15 +214,15 @@ class FolderKnowledgeRepository(KnowledgeRepository):
         raise NotImplementedError
 
     def _kp_new_revision(self, path, uuid=None):
-        self._kp_write_ref(path, "REVISION", encode(self._kp_get_revision(path) + 1))
+        self._kp_write_ref(path, 'REVISION', encode(
+            self._kp_get_revision(path) + 1))
         if uuid:
-            self._kp_write_ref(path, "UUID", encode(uuid))
+            self._kp_write_ref(path, 'UUID', encode(uuid))
 
     def _kp_read_ref(self, path, reference, revision=None):
         path = os.path.join(self.path, path)
         if os.path.isdir(path):
-            with open(os.path.join(self.path, path, reference), 'rb') as f:
-                return f.read()
+            return read_binary(os.path.join(self.path, path, reference))
         else:
             kp = KnowledgePost.from_file(path, format='kp')
             return kp._read_ref(reference)
