@@ -1,15 +1,14 @@
-import os
-import shutil
-import logging
-import re
-import socket
-from io import open
-
-import git
-import yaml
-
 from ..repository import KnowledgeRepository
 from ..utils.encoding import encode
+from ..utils.files import get_path
+from io import open
+from knowledge_repo.utils.files import read_binary, write_binary
+import git
+import logging
+import os
+import re
+import shutil
+import yaml
 
 logger = logging.getLogger(__name__)
 
@@ -18,8 +17,10 @@ class GitKnowledgeRepository(KnowledgeRepository):
     _registry_keys = ['git']
 
     TEMPLATES = {
-        'README.md': os.path.abspath(os.path.join(os.path.dirname(__file__), '../templates', 'repository_readme.md')),
-        '.knowledge_repo_config.yml': os.path.abspath(os.path.join(os.path.dirname(__file__), '../templates', 'repository_config.yml'))
+        'README.md': get_path(
+            __file__, '../templates', 'repository_readme.md'),
+        '.knowledge_repo_config.yml': get_path(
+            __file__, '../templates', 'repository_config.yml'),
     }
 
     @classmethod
@@ -28,12 +29,18 @@ class GitKnowledgeRepository(KnowledgeRepository):
         if os.path.exists(path):
             try:
                 repo = git.Repo(path)
-                logger.warning("Repository already exists for uri '{}'. Checking if configuration is needed...".format(uri))
+                logger.warning(
+                    f"Repository already exists for uri '{uri}'. "
+                    "Checking if configuration is needed...")
             except git.InvalidGitRepositoryError:
                 if os.path.isdir(path):
-                    logger.warning("Upgrading existing directory at '{}' to a git knowledge repository...".format(path))
+                    logger.warning(
+                        f"Upgrading existing directory at '{path}' "
+                        "to a git knowledge repository...")
                 else:
-                    raise RuntimeError("File exists at nominated path: {}. Cannot proceed with repository initialization.".format(path))
+                    raise RuntimeError(
+                        f"File exists at nominated path: {path}. "
+                        "Cannot proceed with repository initialization.")
 
         repo = git.Repo.init(path, mkdir=True)
 
@@ -47,14 +54,15 @@ class GitKnowledgeRepository(KnowledgeRepository):
                 repo.index.add([filename])
                 added_files += 1
             else:
-                logger.warning("Not overriding existing file '{}'.".format(filename))
+                logger.warning(f"Not overriding existing file '{filename}'.")
 
         if added_files > 0:
             repo.index.commit("Initial creation of knowledge repository.")
 
         return GitKnowledgeRepository(path)
 
-    def init(self, config='git:///.knowledge_repo_config.yml', auto_create=False):
+    def init(self, config='git:///.knowledge_repo_config.yml',
+             auto_create=False):
         self.config.update_defaults(published_branch='master')
         self.config.update_defaults(remote_name='origin')
         self.auto_create = auto_create
@@ -64,21 +72,25 @@ class GitKnowledgeRepository(KnowledgeRepository):
         try:
             self.git_read('.knowledge_repo_config.py')
             logger.warning(
-                "This knowledge repository has a legacy configuration file that "
-                "will not be loaded due to security issues "
+                "This knowledge repository has a legacy configuration file "
+                "that will not be loaded due to security issues "
                 "(.knowledge_repo_config.py). This may lead to unexpected "
                 "behavior. Please talk to your local Knowledge Repo admins "
                 "for advice if you are unsure."
             )
-        except:
+        except Exception as e:
+            print(f'Exception encountered: {e}')
             pass
 
         if config.startswith('git:///'):
-            assert config.endswith('.yml'), "In-repository configuration must be a YAML file."
+            assert config.endswith(
+                '.yml'), "In-repository configuration must be a YAML file."
             try:
-                self.config.update(yaml.safe_load(self.git_read(config.replace('git:///', ''))))
+                self.config.update(yaml.safe_load(
+                    self.git_read(config.replace('git:///', ''))))
             except KeyError:
-                logger.warning("Repository missing configuration file: {}".format(config))
+                logger.warning(
+                    f"Repository missing configuration file: {config}")
         else:
             self.config.update(config)
 
@@ -95,9 +107,9 @@ class GitKnowledgeRepository(KnowledgeRepository):
             if self.auto_create:
                 self.create(path)
             else:
-                raise ValueError("Provided path '{}' does not exist.".format(path))
+                raise ValueError(f"Provided path '{path}' does not exist.")
         assert self.__is_valid_repo(
-            path), "Provided path '{}' is not a valid repository.".format(path)
+            path), f"Provided path '{path}' is not a valid repository."
         self._path = path
         self.uri = path  # Update uri to point to absolute path of repository
 
@@ -124,22 +136,26 @@ class GitKnowledgeRepository(KnowledgeRepository):
             return self.git.remote(self.config.remote_name)
         return None
 
-    # ----------- Repository actions / state ------------------------------------
+    # ----------- Repository actions / state ----------------------------------
     @property
     def revision(self):
         c = self.git.commit()
-        return "{}_{}".format(str(c.committed_date), c.hexsha)
+        return f'{c.committed_date}_{c.hexsha}'
 
     def update(self, branch=None):
         branch = branch or self.config.published_branch
         if not self.git_has_remote:
             return
-        if not self.__remote_available:
-            logger.warning("Cannot connect to remote repository hosted on {}. Continuing locally with potentially outdated code.".format(
-                self.__remote_host))
-            return
+
         logger.info("Fetching updates to the knowledge repository...")
-        self.git_remote.fetch()
+        try:
+            self.git_remote.fetch()
+        except git.exc.GitCommandError:
+            logger.warning('Cannot fetch from remote repository hosted '
+                           f'on {self.__remote_host}. Continuing locally '
+                           'with potentially outdated code.')
+            return
+
         current_branch = self.git.active_branch
         self.git.branches[branch].checkout()
         self.git_remote.pull(branch)
@@ -160,9 +176,11 @@ class GitKnowledgeRepository(KnowledgeRepository):
     @property
     def status_message(self):
         status = self.status
-        message = "Currently checked out on the `{branch}` branch.".format(branch=status['branch'])
+        branch = status['branch']
+        message = f"Currently checked out on the `{branch}` branch."
         if len(status['changed_files']) > 0:
-            message += "Files modified: \n {modified}".format(modified='\n\t- '.join(status['changed_files']))
+            modified = '\n\t- '.join(status['changed_files'])
+            message += f'Files modified: \n {modified}'
         return message
 
     # ---------------- Git properties and actions -------------------------
@@ -172,11 +190,11 @@ class GitKnowledgeRepository(KnowledgeRepository):
         if prefix is not None:
             tree = tree[prefix]
         return [o.path for o in tree.traverse(
-                prune=lambda i, d: isinstance(i, git.Submodule) or os.path.dirname(i.path).endswith('.kp'),
+                prune=lambda i, d: isinstance(i, git.Submodule) or
+                os.path.dirname(i.path).endswith('.kp'),
                 visit_once=False,
                 predicate=lambda i, d: i.path.endswith('.kp')
-                )
-                ]
+                )]
 
     def git_read(self, path, commit=None):
         commit = self.git.commit(commit or self.config.published_branch)
@@ -185,7 +203,8 @@ class GitKnowledgeRepository(KnowledgeRepository):
     @property
     def git_local_branches(self):
         unmerged_branches = [branch.replace(
-            '*', '').strip() for branch in self.git.git.branch('--no-merged', self.config.published_branch).split('\n')]
+            '*', '').strip() for branch in self.git.git.branch(
+            '--no-merged', self.config.published_branch).split('\n')]
         return unmerged_branches
 
     def __get_path_from_ref(self, ref):
@@ -202,7 +221,8 @@ class GitKnowledgeRepository(KnowledgeRepository):
             branches = self.git_local_branches
         posts = {}
         for branch in branches:
-            posts[branch] = set([self.__get_path_from_ref(diff.a_path) for diff in self.git_diff(branch)])
+            posts[branch] = set([self.__get_path_from_ref(diff.a_path)
+                                for diff in self.git_diff(branch)])
             posts[branch].discard(None)
 
         if not as_dict:
@@ -233,15 +253,17 @@ class GitKnowledgeRepository(KnowledgeRepository):
 
         # Deal with ambiguity
         if interactive:
-            print("There are multiple branches for post '{}'.".format(path))
+            print(f"There are multiple branches for post '{path}'.")
             for i, branch in enumerate(branches):
-                print("{}. {}".format(i, branch))
+                print(f'{i}. {branch}')
             response = None
             while not isinstance(response, int):
-                response = input('Please select the branch you would like to use: ')
+                response = input(
+                    'Please select the branch you would like to use: ')
                 try:
                     response = int(response)
-                except:
+                except Exception as e:
+                    print(f'Exception encountered: {e}')
                     response = None
         else:
             response = 0
@@ -255,12 +277,14 @@ class GitKnowledgeRepository(KnowledgeRepository):
             return self.git.active_branch
 
         if not isinstance(branch, str):
-            raise ValueError("'{}' of type `{}` is not a valid branch descriptor.".format(branch, type(branch)))
+            raise ValueError(f"'{branch}' of type `{type(branch)}` "
+                             "is not a valid branch descriptor.")
 
         try:
             return self.git.branches[branch]
         except IndexError:
-            raise ValueError("Specified branch `{}` does not exist.".format(branch))
+            raise ValueError(
+                f"Specified branch `{branch}` does not exist.")
 
     def git_checkout(self, branch, soft=False, reset=False, create=False):
         if not create:
@@ -268,10 +292,16 @@ class GitKnowledgeRepository(KnowledgeRepository):
             branch_obj.checkout()
             return branch_obj
 
-        if soft and self.git.active_branch.name not in [self.config.published_branch, branch] and not self.git.active_branch.name.endswith('.kp'):
+        if soft and self.git.active_branch.name not in \
+                [self.config.published_branch, branch] and not \
+                self.git.active_branch.name.endswith('.kp'):
             response = None
             while response not in ['y', 'n']:
-                response = input('It looks like you have checked out the `{}` branch, whereas we were expecting to use `{}`. Do you want to use your current branch instead? (y/n) '.format(self.git.active_branch.name, branch))
+                active_branch = self.git.active_branch.name
+                response = input(
+                    f"It looks like you have checked out the `{active_branch}`"
+                    f" branch, whereas we were expecting to use `{branch}`. Do"
+                    " you want to use your current branch instead? (y/n) ")
                 if response == 'y':
                     branch = self.git.active_branch.name
 
@@ -283,10 +313,12 @@ class GitKnowledgeRepository(KnowledgeRepository):
                         ref_head = ref
                         break
             if not ref_head:
-                ref_head = self.git_remote.refs.master if self.git_has_remote else self.git.branches.master
+                ref_head = self.git_remote.refs.master \
+                    if self.git_has_remote else self.git.branches.master
             else:
                 logger.warning(
-                    "The branch `{}` already exists as upstream, and you maybe clobbering someone's work. Please check.".format(ref_head.name))
+                    f"The branch `{ref_head.name}` already exists as upstream,"
+                    " and you maybe clobbering someone's work. Please check.")
             branch = self.git.create_head(branch, ref_head, force=True)
         else:
             branch = self.git_branch(branch)
@@ -295,7 +327,8 @@ class GitKnowledgeRepository(KnowledgeRepository):
 
     def git_diff(self, ref=None, ref_base=None, patch=False):
         commit = self.git.commit(ref)
-        ref = self.git.merge_base(self.git.commit(ref_base or self.config.published_branch), commit)[0]
+        ref = self.git.merge_base(self.git.commit(
+            ref_base or self.config.published_branch), commit)[0]
         return commit.diff(ref, create_patch=patch)
 
     # ---------------- Post retrieval methods --------------------------------
@@ -308,73 +341,106 @@ class GitKnowledgeRepository(KnowledgeRepository):
 
         for status in statuses:
             if status == self.PostStatus.PUBLISHED:
-                posts.update(self.git_dir(prefix=prefix, commit=self.config.published_branch))
+                posts.update(self.git_dir(
+                    prefix=prefix, commit=self.config.published_branch))
             else:
                 for branch in local_posts:
                     for post_path in local_posts[branch]:
-                        if prefix is not None and not post_path.startswith(prefix):
+                        if prefix is not None and \
+                                not post_path.startswith(prefix):
                             continue
-                        if self._kp_status(post_path, branch=branch) in statuses:
+                        if self._kp_status(
+                                post_path, branch=branch) in statuses:
                             posts.add(post_path)
         for post in sorted(posts):
             yield post
 
     # ------------- Post submission / addition user flow ----------------------
-    def _add_prepare(self, kp, path, update=False, branch=None, squash=False, message=None):
+    def _add_prepare(self, kp, path, update=False,
+                     branch=None, squash=False, message=None):
         target = os.path.abspath(os.path.join(self.path, path))
         if self.git_has_remote:
             branch = branch or path
         else:
-            logger.warning("This repository does not have a remote, and so post review is being skipped. Adding post directly into published branch...")
+            logger.warning(
+                "This repository does not have a remote, so post review is "
+                "being skipped. Adding post directly into published branch...")
             branch = self.config.published_branch
 
         # Create or checkout the appropriate branch for this project
-        logger.info("Checking out (and/or creating) a new branch `{}`...".format(branch))
-        branch_obj = self.git_checkout(branch, soft=True, reset=squash, create=True)
+        logger.info(
+            f"Checking out (and/or creating) a new branch `{branch}`...")
+        branch_obj = self.git_checkout(
+            branch, soft=True, reset=squash, create=True)
         branch = branch_obj.name
 
-        # Verify that post path does not exist (unless we are updating the post)
-        assert update or not os.path.exists(target), "A knowledge post already exists at '{}'! If you wanted to update it, please pass the '--update' flag.".format(path)
+        # Verify that post path does not exist (unless we are updating
+        # the post)
+        assert update or not os.path.exists(
+            target), f"A knowledge post already exists at '{path}'! " + \
+            "If you wanted to update it, please pass the '--update' flag."
 
         # Add knowledge post to local branch
-        logger.info("Adding and committing '{}' to local branch `{}`...".format(path, branch))
+        logger.info(
+            f"Adding and committing '{path}' to local branch `{branch}`...")
 
-    def _add_cleanup(self, kp, path, update=False, branch=None, squash=False, message=None):
+    def _add_cleanup(self, kp, path, update=False,
+                     branch=None, squash=False, message=None):
         self.git.index.add([path])
 
         # Commit the knowledge post and rollback if it fails
         try:
             if message is None:
-                message = input("Please enter a commit message for this post: ")
+                message = input(
+                    "Please enter a commit message for this post: ")
             self.git.index.commit(message)
         except (KeyboardInterrupt, Exception) as e:
             if message is None:
-                logger.warning("No commit message input for post '{}'. Rolling back post addition...")
+                logger.warning(
+                    f"No commit message input for post '{path}'. "
+                    "Rolling back post addition...")
             else:
-                logger.error("Something went wrong. Rolling back post addition...")
+                logger.error(
+                    "Something went wrong. Rolling back post addition...")
             self.git.index.reset()
             try:
                 self.git.git.clean('-df', path)
                 self.git.git.checkout('--', path)
-            except:
+            except Exception as e:
+                print(f'Exception encountered: {e}')
                 pass
             raise e
 
     def _submit(self, path=None, branch=None, force=False):
         if not self.git_has_remote:
-            raise RuntimeError("Could not find remote repository `{}` into which this branch should be submitted.".format(self.config.remote_name))
+            remote_name = self.config.remote_name
+            raise RuntimeError(
+                f"Could not find remote repository `{remote_name}` into "
+                "which this branch should be submitted.")
         if branch is None and path is None:
-            raise ValueError("To submit a knowledge post, a path to the post and/or a git branch must be specified.")
+            raise ValueError(
+                "To submit a knowledge post, a path to the post "
+                "and/or a git branch must be specified.")
         if branch is None:
             branch = self.git_branch_for_post(path)
         if branch is None:
-            raise ValueError("It does not appear that you have any drafts in progress for '{}'.".format(path))
+            raise ValueError("It does not appear that you have "
+                             f"any drafts in progress for '{path}'.")
 
-        if not self.__remote_available:
-            raise RuntimeError("Cannot connect to remote repository {} ({}). Please check your connection, and then try again.".format(self.config.remote_name, self.git_remote.url))
+        try:
+            self.git_remote.push(branch, force=force)
+        except git.exc.GitCommandError as e:
+            remote_name = self.config.remote_name
+            remote_url = self.git_remote.url
+            err = e.stderr.strip()
+            raise RuntimeError(
+                f"Failed to push to remote repository {remote_name} "
+                f"({remote_url}). Please check the following error, "
+                f"and then try again:\n\n{err}")
 
-        self.git_remote.push(branch, force=force)
-        logger.info("Pushed local branch `{}` to upstream branch `{}`. Please consider starting a pull request, or otherwise merging into master.".format(branch, branch))
+        logger.info(f"Pushed local branch `{branch}` to upstream branch "
+                    f"`{branch}`. Please consider starting a pull request, "
+                    "or otherwise merging into master.")
 
     def _publish(self, path):  # Publish a post for general perusal
         raise NotImplementedError
@@ -388,20 +454,24 @@ class GitKnowledgeRepository(KnowledgeRepository):
     def _remove(self, path, all=False):
         raise NotImplementedError
 
-    # ------------ Knowledge Post Data Retrieval Methods -------------------------
+    # ------------ Knowledge Post Data Retrieval Methods ----------------------
 
     def _kp_uuid(self, path):
         try:
             return self._kp_read_ref(path, 'UUID')
-        except:
+        except Exception as e:
+            print(f'Exception encountered: {e}')
             return None
 
     def _kp_path(self, path, rel=None):
-        return KnowledgeRepository._kp_path(self, os.path.expanduser(path), rel=rel or self.path)
+        return KnowledgeRepository._kp_path(
+            self, os.path.expanduser(path), rel=rel or self.path)
 
     def _kp_exists(self, path, revision=None):
-        # For speed, first check whether it exists in the checked out branch, then search more deeply
-        return os.path.isdir(os.path.join(self.path, path)) or (self.git_branch_for_post(path, interactive=False) is not None)
+        # For speed, first check whether it exists in the
+        # checked out branch, then search more deeply
+        return os.path.isdir(os.path.join(self.path, path)) or \
+            (self.git_branch_for_post(path, interactive=False) is not None)
 
     def _kp_status(self, path, revision=None, detailed=False, branch=None):
         if not hasattr(self, '_dir_cache'):
@@ -414,19 +484,21 @@ class GitKnowledgeRepository(KnowledgeRepository):
             branch = self.git_branch(branch)
 
         if branch is None:
-            return ValueError("No such post: {}".format(path))
+            return ValueError(f'No such post: {path}')
 
         if branch.name == self.config.published_branch:
             status = self.PostStatus.PUBLISHED, None
         elif self.git_has_remote and branch.name in self.git_remote.refs:
             remote_branch = self.git_remote.refs[branch.name].name
-            behind = len(list(self.git.iter_commits('{}..{}'.format(branch, remote_branch))))
-            ahead = len(list(self.git.iter_commits('{}..{}'.format(remote_branch, branch))))
+            behind = len(list(self.git.iter_commits(
+                f'{branch}..{remote_branch}')))
+            ahead = len(list(self.git.iter_commits(
+                f'{remote_branch}..{branch}')))
 
             status = (self.PostStatus.SUBMITTED,
-                      (" - {} commits behind".format(behind) if behind else '') +
-                      (" - {} commits ahead".format(ahead) if ahead else '') +
-                      (" [On branch: {}]".format(branch) if branch != path else ''))
+                      (f' - {behind} commits behind' if behind else '') +
+                      (f' - {ahead} commits ahead' if ahead else '') +
+                      (f' [On branch: {branch}]' if branch != path else ''))
         else:
             status = self.PostStatus.DRAFT, None
 
@@ -435,18 +507,20 @@ class GitKnowledgeRepository(KnowledgeRepository):
         return status[0]
 
     def _kp_get_revision(self, path):
-        # We use a 'REVISION' file in the knowledge post folder rather than using git
-        # revisions because using git rev-parse is slow.
+        # We use a 'REVISION' file in the knowledge post folder rather
+        # than using git revisions because using git rev-parse is slow.
         try:
             return int(self._kp_read_ref(path, 'REVISION'))
-        except:
+        except Exception as e:
+            print(f'Exception encountered: {e}')
             return 0
 
     def _kp_get_revisions(self, path):  # slow
         # TODO: In the future, we may want to use something like:
-        #    self.git.iter_commits(paths=os.path.join(self.path, path, 'knowledge.md'))
-        # But this will require a lot of piping and may not make sense in the context
-        # of a non-bare git repository.
+        #    self.git.iter_commits(paths=os.path.join(
+        #                          self.path, path, 'knowledge.md'))
+        # But this will require a lot of piping and may not make sense
+        # in the context of a non-bare git repository.
         raise NotImplementedError
 
     def _kp_write_ref(self, path, reference, data, uuid=None, revision=None):
@@ -454,32 +528,36 @@ class GitKnowledgeRepository(KnowledgeRepository):
         ref_dir = os.path.dirname(ref_path)
         if not os.path.exists(ref_dir):
             os.makedirs(ref_dir)
-        with open(ref_path, 'wb') as f:
-            return f.write(data)
+        write_binary(ref_path, data)
 
-    def _kp_dir(self, path, parent=None, revision=None):  # TODO: Account for revision
+    # TODO: Account for revision
+    def _kp_dir(self, path, parent=None, revision=None):
         if parent:
             path = os.path.join(path, parent)
-        for dirpath, dirnames, filenames in os.walk(os.path.join(self.path, path)):
+        for dirpath, dirnames, filenames in os.walk(
+                os.path.join(self.path, path)):
             for filename in filenames:
                 if dirpath == "" and filename == "REVISION":
                     continue
-                yield os.path.relpath(os.path.join(dirpath, filename), os.path.join(self.path, path))
+                yield os.path.relpath(
+                    os.path.join(dirpath, filename),
+                    os.path.join(self.path, path))
 
-    def _kp_has_ref(self, path, reference, revision=None):  # TODO: Account for revision
+    # TODO: Account for revision
+    def _kp_has_ref(self, path, reference, revision=None):
         return os.path.isfile(os.path.join(self.path, path, reference))
 
     def _kp_diff(self, path, head, base):
         raise NotImplementedError
 
     def _kp_new_revision(self, path, uuid=None):
-        self._kp_write_ref(path, "REVISION", encode(self._kp_get_revision(path) + 1))
+        self._kp_write_ref(path, "REVISION", encode(
+            self._kp_get_revision(path) + 1))
         if uuid:
             self._kp_write_ref(path, "UUID", encode(uuid))
 
     def _kp_read_ref(self, path, reference, revision=None):
-        with open(os.path.join(self.path, path, reference), 'rb') as f:
-            return f.read()
+        return read_binary(os.path.join(self.path, path, reference))
 
     # ------------- Utility methods --------------------------------------
     def __abspath(self, path):
@@ -503,21 +581,3 @@ class GitKnowledgeRepository(KnowledgeRepository):
                 if m.group(3):
                     port = m.group(3)
         return int(port)
-
-    @property
-    def __remote_available(self):
-        # TODO: support more types of hosts
-        host = self.__remote_host
-        port = self.__remote_port
-
-        if host:
-            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            s.settimeout(0.5)
-            try:
-                s.connect((socket.gethostbyname(host), port))
-                return True
-            except:
-                return False
-            finally:
-                s.close()
-        return True
