@@ -1,6 +1,5 @@
-import posixpath
-
 from ..repository import KnowledgeRepository
+import posixpath
 
 
 class MetaKnowledgeRepository(KnowledgeRepository):
@@ -10,21 +9,25 @@ class MetaKnowledgeRepository(KnowledgeRepository):
         pass
 
     def __repos_for_prefix(self, path_prefix=None):
-        repo_prefixes = sorted(self.uri.keys(), key=lambda x: len(x), reverse=True)
+        repo_prefixes = sorted(
+            self.uri.keys(), key=lambda x: len(x), reverse=True)
         for repo_prefix in repo_prefixes:
             if path_prefix is None or path_prefix.startswith(repo_prefix):
-                repo_prefix, repo, repo_path = self.__repo_for_path(path_prefix or repo_prefix)
+                repo_prefix, repo, repo_path = self.__repo_for_path(
+                    path_prefix or repo_prefix)
                 repo_path = None if not repo_path else repo_path
                 yield repo_prefix, repo, repo_path
 
     def __repo_for_path(self, path):
-        path_prefixes = sorted(self.uri.keys(), key=lambda x: len(x), reverse=True)
+        path_prefixes = sorted(
+            self.uri.keys(), key=lambda x: len(x), reverse=True)
         for prefix in path_prefixes:
             if path.startswith(prefix):
-                relpath = posixpath.relpath(path, prefix or '.') if path else ''
+                relpath = posixpath.relpath(
+                    path, prefix or '.') if path else ''
                 return prefix, self.uri[prefix], relpath if relpath != '.' else None
-        raise ValueError(("No KnowledgeRepository found for '{}', "
-                          "paths must be prefixed with {}.").format(path, path_prefixes))
+        raise ValueError(f"No KnowledgeRepository found for '{path}', "
+                         f"paths must be prefixed with {path_prefixes}.")
 
     def __distribute_method(self, method, **kwargs):
         return {name: getattr(repo, method)(**kwargs) for name, repo in list(self.uri.items())}
@@ -101,7 +104,8 @@ class MetaKnowledgeRepository(KnowledgeRepository):
 
     def _kp_repository_uri(self, path):
         prefix, repository, repo_path = self.__repo_for_path(path)
-        return '{{{prefix}}}{repository_uri}'.format(prefix=prefix, repository_uri=repository._kp_repository_uri(repo_path))
+        repository_uri = repository._kp_repository_uri(repo_path)
+        return f'{{{prefix}}}{repository_uri}'
 
     def _kp_uuid(self, path):
         return self.__delegate_for_path(path, '_kp_uuid')
