@@ -1,17 +1,16 @@
-import sys
-import os
-import posixpath
-from abc import abstractmethod, abstractproperty
-import datetime
-from collections import OrderedDict
-from enum import Enum
-from urllib.parse import urlparse
-
 from . import config_defaults
-from .post import KnowledgePost
 from .config import KnowledgeRepositoryConfig
+from .constants import KP_EXTENSION
+from .post import KnowledgePost
 from .postprocessor import KnowledgePostProcessor
 from .utils.registry import SubclassRegisteringABCMeta
+from abc import abstractmethod, abstractproperty
+from collections import OrderedDict
+from datetime import datetime
+from enum import Enum
+from urllib.parse import urlparse
+import os
+import posixpath
 
 
 class KnowledgeRepository(object, metaclass=SubclassRegisteringABCMeta):
@@ -37,7 +36,8 @@ class KnowledgeRepository(object, metaclass=SubclassRegisteringABCMeta):
 
     @classmethod
     def for_uris(cls, uri):
-        # Import this within this method so as not to cause import resolution problems
+        # Import this within this method so as not to cause import
+        # resolution problems
         from .repositories.meta import MetaKnowledgeRepository
         if isinstance(uri, str):
             uris = {'': uri}
@@ -78,7 +78,8 @@ class KnowledgeRepository(object, metaclass=SubclassRegisteringABCMeta):
 
     @config.setter
     def config(self, config):
-        assert isinstance(config, KnowledgeRepositoryConfig), "`config` should be a `KnowledgeRepositoryConfig` instance."
+        assert isinstance(config, KnowledgeRepositoryConfig), \
+            '`config` should be a `KnowledgeRepositoryConfig` instance.'
         self._config = config
 
     @property
@@ -100,23 +101,24 @@ class KnowledgeRepository(object, metaclass=SubclassRegisteringABCMeta):
                     uri_dict[parent] = uri
                 elif isinstance(uri, dict):
                     for mountpoint, u in uri.items():
-                        add_uris(uri_dict, u, parent=posixpath.join(parent, mountpoint))
+                        add_uris(uri_dict, u, parent=posixpath.join(
+                            parent, mountpoint))
                 elif isinstance(uri, KnowledgeRepository):
                     for mountpoint, u in uri.uris.items():
                         uri_dict[posixpath.join(parent, mountpoint)] = u
                 else:
-                    raise ValueError("Unrecognised uri: {}".format(uri))
+                    raise ValueError(f'Unrecognised uri: {uri}')
 
             add_uris(uri_dict, self.uri)
             return uri_dict
 
-        raise ValueError("Unrecognised KnowledgeRepository.uri: {}".format(self.uri))
+        raise ValueError(f'Unrecognised KnowledgeRepository.uri: {self.uri}')
 
     @property
     def revisions(self):
-        # This method provides a mapping from uri to revision for this repository
-        # and/or any nested repositories. This is most useful when checking if an
-        # update is required server side.
+        # This method provides a mapping from uri to revision for this
+        # repository and/or any nested repositories. This is most useful
+        # when checking if an update is required server side.
         if isinstance(self.uri, str):
             return {self.uri: self.revision}
 
@@ -125,21 +127,22 @@ class KnowledgeRepository(object, metaclass=SubclassRegisteringABCMeta):
 
             def add_revisions(revision_dict, uri):
                 if isinstance(uri, str):
-                    revision_dict[uri] = KnowledgeRepository.for_uri(uri).revision
+                    revision_dict[uri] = \
+                        KnowledgeRepository.for_uri(uri).revision
                 elif isinstance(uri, dict):
                     for u in uri.values():
                         add_revisions(revision_dict, u)
                 elif isinstance(uri, KnowledgeRepository):
                     revision_dict.update(uri.revisions)
                 else:
-                    raise ValueError("Unrecognised uri: {}".format(uri))
+                    raise ValueError(f'Unrecognised uri: {uri}')
 
             add_revisions(revision_dict, self.uris)
             return revision_dict
 
-        raise ValueError("Unrecognised KnowledgeRepository.uri: {}".format(self.uri))
+        raise ValueError(f'Unrecognised KnowledgeRepository.uri: {self.uri}')
 
-    # ------------- Repository actions / state ------------------------------------
+    # ------------- Repository actions / state --------------------------------
 
     def session_begin(self):
         pass
@@ -165,31 +168,39 @@ class KnowledgeRepository(object, metaclass=SubclassRegisteringABCMeta):
     def set_active_draft(self, path):
         pass
 
-    # -------------- Post retrieval methods --------------------------------------
+    # -------------- Post retrieval methods ----------------------------------
 
     def post(self, path, revision=None):
         if path is None:
-            raise ValueError("path is None")
+            raise ValueError('path is None')
         path = self._kp_path(path)
-        if not self.has_post(path, revision=revision) and path in self.config.aliases:
+        if not self.has_post(path, revision=revision) \
+                and path in self.config.aliases:
             path = self.config.aliases[path]
             if path in self.config.alias:
-                raise ValueError("Alias cycle detected.")
-        assert self.has_post(path, revision=revision), "{} does not have a post for path '{}'.".format(self.__class__.__name__, path)
-        return KnowledgePost(path=path, repository=self, revision=revision or self._kp_get_revision(path))
+                raise ValueError('Alias cycle detected.')
+        assert self.has_post(path, revision=revision), \
+            f"{self.__class__.__name__} does not have a post at '{path}'."
+        return KnowledgePost(
+            path=path, repository=self,
+            revision=revision or self._kp_get_revision(path))
 
     def dir(self, prefix=None, status=None):
         if prefix is None or isinstance(prefix, str):
             prefixes = [prefix]
         else:
             prefixes = prefix
-        assert all([prefix is None or isinstance(prefix, str) for prefix in prefixes]), "All path prefixes must be strings."
-        prefixes = [prefix if prefix is None else posixpath.relpath(prefix) for prefix in prefixes]
+        assert all([prefix is None or isinstance(prefix, str) for prefix
+                    in prefixes]), 'All path prefixes must be strings.'
+        prefixes = [prefix if prefix is None else posixpath.relpath(prefix)
+                    for prefix in prefixes]
         if isinstance(status, str):
             if status == 'all':
-                status = [self.PostStatus.DRAFT, self.PostStatus.SUBMITTED, self.PostStatus.PUBLISHED, self.PostStatus.UNPUBLISHED]
+                status = \
+                    [self.PostStatus.DRAFT, self.PostStatus.SUBMITTED,
+                     self.PostStatus.PUBLISHED, self.PostStatus.UNPUBLISHED]
             else:
-                raise ValueError('Status alias `{}` not recognised.'.format(status))
+                raise ValueError(f'Status alias `{status}` not recognised.')
         if status is not None and not isinstance(status, list):
             status = [status]
         elif status is None:
@@ -208,10 +219,12 @@ class KnowledgeRepository(object, metaclass=SubclassRegisteringABCMeta):
         return self._kp_exists(self._kp_path(path), revision=revision)
 
     def post_status(self, path, revision=None, detailed=False):
-        return self._kp_status(self._kp_path(path), revision=revision, detailed=detailed)
+        return self._kp_status(self._kp_path(path), revision=revision,
+                               detailed=detailed)
 
     def post_statuses(self, paths, detailed=False):
-        return OrderedDict([(path, self.post_status(path, detailed=detailed)) for path in paths])
+        return OrderedDict([(path, self.post_status(path, detailed=detailed))
+                            for path in paths])
 
     def posts(self, status=None, only_valid=False):
         for path in self.dir(status=status):
@@ -234,22 +247,31 @@ class KnowledgeRepository(object, metaclass=SubclassRegisteringABCMeta):
 
     # -------------- Post submission / addition user flow --------------------
 
-    def add(self, kp, path=None, update=False, **kwargs):  # Create a new knowledge post draft
-        assert isinstance(kp, KnowledgePost), "One can only add KnowledgePost objects to a KnowledgeRepository."
+    # Create a new knowledge post draft
+    def add(self, kp, path=None, update=False, **kwargs):
+        assert isinstance(kp, KnowledgePost), \
+            'One can only add KnowledgePost objects to a KnowledgeRepository.'
         path = path or kp.path
         if not path:
-            raise ValueError("Post path not provided for Knowledge Post, and one is not specified within the knowledge post. Either add the path to post headers using `path: <path>` or specify the project path on the command line adding `-p <path>` to the current command.")
+            raise ValueError(
+                'Post path not provided for Knowledge Post, and one is not '
+                'specified within the knowledge post. Either add the path to '
+                'post headers using `path: <path>` or specify the project '
+                'path on the command line adding `-p <path>` to the current '
+                'command.')
         path = self._kp_path(path)
         path = self.config.path_parse(path)
 
-        current_datetime = datetime.datetime.now()
+        now = datetime.now()
         authors = kp.headers['authors']
-        new_authors = [self.config.username_parse(author) for author in authors]
-        if new_authors != authors or kp.headers['updated_at'] < current_datetime:
-            kp.update_headers(authors=new_authors, updated_at=current_datetime)
+        new_authors = [self.config.username_parse(
+            author) for author in authors]
+        if new_authors != authors or kp.headers['updated_at'] < now:
+            kp.update_headers(authors=new_authors, updated_at=now)
 
         for postprocessor, postprocessor_kwargs in self.config.postprocessors:
-            KnowledgePostProcessor._get_subclass_for(postprocessor)(**postprocessor_kwargs).process(kp)
+            KnowledgePostProcessor._get_subclass_for(
+                postprocessor)(**postprocessor_kwargs).process(kp)
 
         cleanup_kwargs = self._add_prepare(kp, path, update, **kwargs)
 
@@ -279,11 +301,11 @@ class KnowledgeRepository(object, metaclass=SubclassRegisteringABCMeta):
     def _submit(self, path):  # Submit a post for review
         raise NotImplementedError
 
-    def accept(self, path):  # Submit a post for review
+    def accept(self, path):  # Accept a post after review
         return self._accept(self._kp_path(path))
 
     @abstractmethod
-    def _accept(self, path):  # Submit a post for review
+    def _accept(self, path):  # Accept a post after review
         raise NotImplementedError
 
     def publish(self, path):  # Publish a post for general perusal
@@ -307,7 +329,7 @@ class KnowledgeRepository(object, metaclass=SubclassRegisteringABCMeta):
     def _remove(self, path, all=False):
         raise NotImplementedError
 
-    # ----------- Knowledge Post Data Retrieval/Pushing Methods --------------------
+    # ----------- Knowledge Post Data Retrieval/Pushing Methods ---------------
 
     def _kp_repository_uri(self, path):
         return self.uri
@@ -322,12 +344,14 @@ class KnowledgeRepository(object, metaclass=SubclassRegisteringABCMeta):
         path = os.path.relpath(os.path.abspath(os.path.join(rel, path)), rel)
         if os.name == 'nt':
             path = path.replace(os.path.sep, os.path.altsep)
-        assert all([not segment.endswith('.kp') for segment in path.split(
-            '/')[:-1]]), "The post path may not contain a directory named '*.kp'."
+        assert all([not segment.endswith(
+            KP_EXTENSION) for segment in path.split('/')[:-1]]), \
+            "The post path may not contain a directory named '*.kp'."
         if path == '.' or path.startswith('..'):
-            raise ValueError("Provided path '{}' is outside of the knowledge repository.".format(path))
-        if not path.endswith('.kp'):
-            path += '.kp'
+            raise ValueError(f"Provided path '{path}' is outside"
+                             " of the knowledge repository.")
+        if not path.endswith(KP_EXTENSION):
+            path += KP_EXTENSION
         return path
 
     @abstractmethod
@@ -375,14 +399,16 @@ class KnowledgeRepository(object, metaclass=SubclassRegisteringABCMeta):
 
     def _kp_save(self, kp, path, update=False):
         if not update and self.has_post(path):
-            raise ValueError("A knowledge post with the same path already exists. To update it, set the update flag.")
+            raise ValueError('A knowledge post with the same path already'
+                             ' exists. To update it, set the update flag.')
         kp.uuid = self._kp_uuid(path) or kp.uuid
         kp.path = path
         kp.revision = self._kp_new_revision(path, uuid=kp.uuid)
         kp.repository = self
 
         for ref in kp._dir():
-            self._kp_write_ref(path, ref, kp._read_ref(ref), uuid=kp.uuid, revision=kp.revision)
+            self._kp_write_ref(path, ref, kp._read_ref(
+                ref), uuid=kp.uuid, revision=kp.revision)
 
     @property
     def web_uri(self):
@@ -391,4 +417,5 @@ class KnowledgeRepository(object, metaclass=SubclassRegisteringABCMeta):
     # ----------- Interface with web app ----------------------------------
     def get_app(self, *args, **kwargs):
         from . import app
-        return self.config.prepare_app(app.KnowledgeFlask(self, *args, **kwargs))
+        return self.config.prepare_app(
+            app.KnowledgeFlask(self, *args, **kwargs))
