@@ -1,25 +1,25 @@
 import unittest
 
 from bs4 import BeautifulSoup
-from sqlalchemy import and_
 
 from knowledge_repo import KnowledgeRepository
-from knowledge_repo.app.proxies import db_session
 from knowledge_repo.app.models import Post
+from knowledge_repo.app.proxies import db_session
 
 
 class PostTest(unittest.TestCase):
 
     def setUp(self):
-        self.repo = KnowledgeRepository.for_uri('tests/test_repo', auto_create=True)
+        self.repo = KnowledgeRepository.for_uri(
+            'tests/test_repo', auto_create=True)
         self.app = self.repo.get_app(config='tests/config_server.py')
         self.client = self.app.test_client()
 
         with self.app.app_context():
             # for this test, pick one post
             post = (db_session.query(Post)
-                              .filter(Post.is_published)
-                              .first())
+                    .filter(Post.is_published)
+                    .first())
             self.post_id = post.id
             self.post_rendered_url = '/post/{}'.format(post.path)
 
@@ -45,15 +45,15 @@ class PostTest(unittest.TestCase):
         pageviews_str = icon[0].text.strip()
 
         with self.app.app_context():
-            post = (db_session.query(Post)
-                              .filter(Post.id == self.post_id)
-                              .first())
+            post = Post.query.get(self.post_id)
             distinct_viewers = post.view_user_count
             pageviews = post.view_count
 
-        db_pageviews_str = "Viewed {pageviews} times by {distinct_viewers} different users".format(**locals())
+        db_pageviews_str = "Viewed {pageviews} times by {distinct_viewers} different users".format(
+            **locals())
 
-        assert db_pageviews_str == pageviews_str, "'{}' is not '{}'!".format(db_pageviews_str, pageviews_str)
+        assert db_pageviews_str == pageviews_str, "'{}' is not '{}'!".format(
+            db_pageviews_str, pageviews_str)
 
     def test03_check_post_metadata(self):
         """
